<?php


class DZSWTLAdmin {

  public $termMetaExtraOptions = array();

  /**
   * DZSWTLAdmin constructor.
   * @param DZSWooTable $dzswtl
   */
  function __construct($dzswtl) {

    $this->dzswtl = $dzswtl;


    $this->termMetaExtraOptions = array(
      array(
        'name' => 'icon',
        'title' => esc_html__('Icon', DZSWTL_ID),
        'description' => esc_html__('main icon of the category', DZSWTL_ID),
        'type' => 'iconselector',
      ),
      array(
        'name' => 'color',
        'title' => esc_html__('Tag Color'),
        'description' => esc_html__('for tag color '),
        'type' => 'color',
      ),
      array(
        'name' => 'image',
        'title' => esc_html__('Image'),
        'description' => esc_html__('image'),
        'type' => 'media-upload',
      ),
    );


    include_once(DZSWTL_PATH.'inc/php/dzs-shared/admin_termMetas.php');
    add_action('admin_head', array($this, 'handle_admin_head'));
    add_action('admin_footer', array($this, 'handle_admin_footer'));
    add_action('admin_menu', array($this, 'handle_admin_menu'));
    add_action('edit_post', array($this, 'handle_edit_post'));
    add_action('save_post', array($this, 'handle_edit_post'));
    add_action('save_post', array($this, 'handle_save_post'));

    add_action('add_meta_boxes', array($this, 'handle_add_meta_boxes'));


    add_action('current_screen', array($this, 'demoData_stopUsersFromChangingPasswords'));


    add_action('product_cat_edit_form_fields', array($this, 'term_meta_fields'), 10, 10);
    add_action('product_attributes_edit_form_fields', array($this, 'term_meta_fields'), 10, 10);
    add_action('edited_product_cat', 'dzs_admin_termMetas_save_custom_meta', 10, 2);
    add_action('edited_product_attributes', 'dzs_admin_termMetas_save_custom_meta', 10, 2);


    if ($this->dzswtl->db_mainoptions['add_taxonomy_meta_taxs']) {
      $taxs = explode(',', $this->dzswtl->db_mainoptions['add_taxonomy_meta_taxs']);
      foreach ($taxs as $tax) {

        add_action($tax . '_edit_form_fields', array($this, 'term_meta_fields'), 10, 10);
        add_action('edited_' . $tax, 'dzs_admin_termMetas_save_custom_meta', 10, 2);
      }
    }




    add_action('admin_footer-edit.php', array($this, 'status_into_inline_edit'));
    add_action('post_submitbox_misc_actions', array($this, 'add_to_post_status_dropdown'));


    add_action('woocommerce_product_option_terms', array($this, 'product_option_terms'), 10, 2);
    add_action('dzswtl_product_attribute_field', array($this, 'attribute_fields'), 10, 3);


    add_filter('product_attributes_type_selector', array($this, 'add_attribute_types'));

    if (function_exists('wc_get_attribute_taxonomies')) {

      $attribute_taxonomies = wc_get_attribute_taxonomies();

      if (empty($attribute_taxonomies)) {
        return;
      }

      foreach ($attribute_taxonomies as $tax) {
        add_action(DZSWTL_QUERY_PRODUCT_ATTRIBUTE_ABBREVIATION . $tax->attribute_name . '_add_form_fields', array($this, 'add_attribute_fields_pa'));
        add_action(DZSWTL_QUERY_PRODUCT_ATTRIBUTE_ABBREVIATION . $tax->attribute_name . '_edit_form_fields', array($this, 'edit_attribute_fields_pa'), 10, 2);
      }
    }
  }

  public function add_attribute_types($types) {
    $types = array_merge($types, $this->dzswtl->attribute_extra_types);
    return $types;
  }


  public function attribute_fields($type, $value, $form) {
    if (in_array($type, array('select', 'text'))) {
      return;
    }

    // Print the open tag of field container
    printf(
      '<%s class="form-field">%s<label for="term-%s">%s</label>%s',
      'edit' == $form ? 'tr' : 'div',
      'edit' == $form ? '<th>' : '',
      esc_attr($type),
      $this->dzswtl->attribute_extra_types[$type],
      'edit' == $form ? '</th><td>' : ''
    );

    if ($type == 'color') {
      ?>
      <input class="color-with-spectrum attribute-type-<?php echo esc_attr($type) ?>" type="text"
             id="term-<?php echo esc_attr($type) ?>" name="<?php echo esc_attr($type) ?>"
             value="<?php echo esc_attr($value) ?>"/>
      <?php
    }

    // Print the close tag of field container
    echo 'edit' == $form ? '</td></tr>' : '</div>';
  }

  public function get_tax_attribute($taxonomy) {
    global $wpdb;

    $attr = substr($taxonomy, 3);
    $attr = $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . "woocommerce_attribute_taxonomies WHERE attribute_name = '$attr'");

    return $attr;
  }

  /**
   * appear on custom taxonomies ?taxonomy=pa_color&tag_ID=47&post_type=product
   * @param $term
   * @param $taxonomy
   */
  public function edit_attribute_fields_pa($term, $taxonomy) {
    $attr = $this->get_tax_attribute($taxonomy);
    $value = get_term_meta($term->term_id, $attr->attribute_type, true);

    do_action('dzswtl_product_attribute_field', $attr->attribute_type, $value, 'edit');


    DZSWTLHelpers::enqueue_script('spectrum');
  }

  /**
   * appear on custom taxonomies ?taxonomy=pa_color&tag_ID=47&post_type=product
   * @param $taxonomy
   */
  public function add_attribute_fields_pa($taxonomy) {
    $attr = $this->get_tax_attribute($taxonomy);


    do_action('dzswtl_product_attribute_field', $attr->attribute_type, '', 'add');



    DZSWTLHelpers::enqueue_script('spectrum');
  }

  public function product_option_terms($taxonomy, $index) {

    if (!array_key_exists($taxonomy->attribute_type, $this->dzswtl->attribute_extra_types)) {
      return;
    }

    $taxonomy_name = wc_attribute_taxonomy_name($taxonomy->attribute_name);
    global $thepostid;
    ?>

    <select multiple="multiple" data-placeholder="<?php esc_attr_e('Select terms', DZSWTL_ID); ?>"
            class="multiselect attribute_values wc-enhanced-select" name="attribute_values[<?php echo $index; ?>][]">
      <?php

      $all_terms = get_terms($taxonomy_name, apply_filters('woocommerce_product_attribute_terms', array('orderby' => 'name', 'hide_empty' => false)));
      if ($all_terms) {
        foreach ($all_terms as $term) {
          echo '<option value="' . esc_attr($term->term_id) . '" ' . selected(has_term(absint($term->term_id), $taxonomy_name, $thepostid), true, false) . '>' . esc_attr(apply_filters('woocommerce_product_attribute_term_name', $term->name, $term)) . '</option>';
        }
      }
      ?>
    </select>
    <button
      class="button plus select_all_attributes dzswtl-woocommerce-filter-attribute--select-all"><?php esc_html_e('Select all', DZSWTL_ID); ?></button>
    <button class="button minus select_no_attributes"><?php esc_html_e('Select none', DZSWTL_ID); ?></button>
    <button class="button fr plus tawcvs_add_new_attribute"
            data-type="<?php echo $taxonomy->attribute_type ?>"><?php esc_html_e('Add new', DZSWTL_ID); ?></button>

    <?php
  }


  function status_into_inline_edit() { // ultra-simple example
    echo "<script>
	jQuery(document).ready( function() {
		jQuery( 'select[name=\"_status\"]' ).append( '<option value=\"dzs_preview\">Grid preview </option>' );
	});
	</script>";
  }

  function add_to_post_status_dropdown() {
    global $post;
    ?>
    <script>
      jQuery(document).ready(function ($) {
        $("select#post_status").append("<option value=\"dzs_preview\" <?php selected(DZSWTL_POST_STATUS_PREVIEW, $post->post_status); ?>>Grid preview</option>");
      });
    </script>
    <?php
  }

  /**
   * this will add the custom meta field to the add new term page
   * @param $term WP_Term
   */
  function term_meta_fields($term) {


    $fontawesomelink = DZSWTL_URL . 'libs/fontawesome/font-awesome.min.css';
    if (defined("DZSWTL_USE_FONTAWESOME_FROM_CDN") && DZSWTL_USE_FONTAWESOME_FROM_CDN == "ON") {
      $fontawesomelink = 'https://' . 'maxcdn' . 'bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css';
    }

    wp_enqueue_style('fontawesome', $fontawesomelink);

    dzs_admin_termMetas_generateOptions($this->termMetaExtraOptions, $term, DZSWTL_URL);


  }

  function handle_add_meta_boxes() {



    add_meta_box('dzswtl_meta_options', esc_html__('Product shop builder settings', DZSWTL_ID), array($this, 'dzswtl_admin_meta_options'), 'product', 'normal');
  }

  function dzswtl_admin_meta_options() {
    global $post, $wp_version;
    ?>
    <div class="dzswtl-meta-con">
      <input type="hidden" name="dzs_nonce" value="<?php echo wp_create_nonce('dzs_nonce'); ?>"/>


      <div class="dzs-setting">
        <?php
        $lab = 'dzswtl_replace_item_link';
        ?>
        <h4><?php echo __('Replace item link ', DZSWTL_ID); ?></h4>
        <?php echo DZSHelpers::generate_input_text($lab, array('class' => 'input-big-image main-thumb', 'def_value' => '', 'seekval' => get_post_meta($post->ID, $lab, true))); ?>
        <?php
        ?>
        <div
          class='sidenote'><?php echo esc_html__('select a thumbnail for the video ( can auto generate if it is an Vimeo or YouTube track )', DZSWTL_ID); ?></div>
      </div>

      <div class="dzs-setting">
        <?php
        $lab = 'dzswtl_extra_classes';
        ?>
        <h4><?php echo __('Extra Classes', DZSWTL_ID); ?></h4>
        <?php echo DZSHelpers::generate_input_text($lab, array('class' => '', 'def_value' => '', 'seekval' => get_post_meta($post->ID, $lab, true))); ?>
        <div
          class='sidenote'><?php echo __('[advanced] some extra classes that you want added to the portfolio item', DZSWTL_ID); ?></div>
      </div>

    </div>

    <?php


    DZSWTLHelpers::enqueue_dzsSelector();
  }


  function demoData_stopUsersFromChangingPasswords() {

    if (defined('DZS_DEMO_MODE') && DZS_DEMO_MODE == 'TRUE') {

      if (is_admin() && current_user_can('manage_options') == false) {
        $url = get_current_screen();


        $sw = false;


        if ($url->base == 'edit-tags') {
          $sw = true;
        }
        if ($url->base == 'post') {
        }
        if ($url->base == 'upload') {
          $sw = true;
        }
        if ($url->base == 'profile') {
          $sw = true;
        }
        if ($url->base == 'edit-comments') {
          $sw = true;
        }


        if ($sw) {
          die(' DEMO MODE - You do not have access to this page, <a href="' . admin_url() . '">return back to dashboard</a>  ');
        }
      }


    }


  }


  function handle_admin_head() {


    global $pagenow, $post;

//        echo $pagenow;

    ?>
    <script><?php
      echo 'window.dzswtl_settings = { base_url: "' . DZSWTL_URL . '",version: "' . DZSWTL_VERSION . '",admin_url: "' . admin_url() . '",site_url : "' . site_url() . '",translate_add_filter: "' . esc_html__("Add filter", DZSWTL_ID) . '",translate_add_shortcode: "' . esc_html__("Add shop", DZSWTL_ID) . '"';


      if ($post) {
        echo ',post_type:"' . $post->post_type . '"';
      }
      if (1) {
        echo ',shortcode_generator_url:"' . admin_url('admin.php?page=' . DZSWTL_ADMIN_PAGENAME_MAINOPTIONS) . '&dzswtl_shortcode_generator=on' . '"';
      }
      if (1) {
        echo ',filter_generator_url:"' . admin_url('admin.php?page=' . DZSWTL_ADMIN_PAGENAME_MAINOPTIONS) . '&dzswtl_filter_generator=on' . '"';
      }
      if (1) {
        echo ',con_shortcode_generator_url:"' . admin_url('admin.php?page=' . DZSWTL_ADMIN_PAGENAME_MAINOPTIONS) . '&dzswtl_con_shortcode_generator=on' . '"';
      }
      if (1) {
        echo ',generate_layout_item_default_margs:\'' . json_encode($this->dzswtl->generate_layout_item_default_margs) . '\'';
      }

      if ($post && $post->post_type == DZSWTL_POST_TYPE) {


      $struct_item_default = str_replace(array("\r", "\r\n", "\n"), '', dzswtl_generate_layer_item());
      $structItemElement_placeholder = str_replace(array("\r", "\r\n", "\n"), '', dzswtl_generate_layer_item(array('type' => 'element', 'background_color' => 'transparent', 'text' => 'insert text here', 'height' => 'auto')));
      $struct_item_container = str_replace(array("\r", "\r\n", "\n"), '', dzswtl_generate_layer_item(array('type' => 'container', 'background_color' => 'transparent', 'height' => 'auto')));


      ?>
      ,
      struct_layer_element: `<?php echo $structItemElement_placeholder; ?>`
        , struct_layer_container
      :
      `<?php echo $struct_item_container; ?>`
      <?php
      }

      $this->dzswtl->cpt_posts = DZSWTLHelpers::get_cpt_posts();

      echo ',cpt_posts: [';
      foreach ($this->dzswtl->cpt_posts as $mainitem) {
        echo '{ value: "' . $mainitem['value'] . '",label: "' . $mainitem['label'] . '",term_id: "' . $mainitem['term_id'] . '" },';
      }
      echo ' ],';


      $cats = DZSWTLHelpers::get_categories_for_select();
      echo 'cats:' . json_encode($cats) . ',';

      echo ' };';

      // -- options for the gutenberg block
      ?>
      window.dzswtl_gutenberg_block_settings = <?php
      echo DZSWTLHelpers::sanitize_config_to_gutenberg(json_encode($this->dzswtl->gutenberg_block_options));
      ?>;
      <?php
      ?> </script><?php



    if ($post && $post->post_type == DZSWTL_POST_TYPE) {


      add_filter('user_can_richedit', '__return_false', 50);

      wp_enqueue_script('jquery.nestable.js', DZSWTL_URL . 'libs/listbuilder/jquery.nestable.js');



      DZSWTLHelpers::enqueue_script();

      DZSWTLHelpers::enqueue_script('spectrum');

      wp_enqueue_script('listbuilder', DZSWTL_URL . 'libs/listbuilder/listbuilder.js');
      wp_enqueue_style('listbuilder', DZSWTL_URL . 'libs/listbuilder/listbuilder.css');

      wp_enqueue_script('dzstaa', DZSWTL_URL . 'libs/dzstabsandaccordions/dzstabsandaccordions.js');
      wp_enqueue_style('dzstaa', DZSWTL_URL . 'libs/dzstabsandaccordions/dzstabsandaccordions.css');
      wp_enqueue_style('dzstooltip', DZSWTL_URL . 'libs/dzstooltip/dzstooltip.css');


      DZSWTLHelpers::enqueue_dzsSelector();
      wp_enqueue_style('dzstoggle', DZSWTL_URL . 'libs/dzstoggle/dzstoggle.css');
      wp_enqueue_script('dzstoggle', DZSWTL_URL . "libs/dzstoggle/dzstoggle.js");


      if (class_exists('DZSZoomSoundsHelper')) {
        DZSZoomSoundsHelper::enqueueMainScrips();
      }


    }

  }


  function handle_admin_footer() {

    global $post, $pagenow;


    wp_enqueue_style('dzswtl-admin', DZSWTL_URL . "admin/admin.css");
    wp_enqueue_script('dzswtl-admin', DZSWTL_URL . "admin/admin.js");
    wp_enqueue_style('dzswtl-admin-global', DZSWTL_URL . "admin/admin_global.css");
    wp_enqueue_script('dzswtl-admin-global', DZSWTL_URL . "admin/admin_global.js");
    DZSWTLHelpers::enqueue_dzsSelector();
    wp_enqueue_style('dzscheckbox', DZSWTL_URL . 'libs/dzscheckbox/dzscheckbox.css');
    if (($pagenow == 'edit.php' && isset($_GET['post_type']) && $_GET['post_type'] == DZSWTL_POST_TYPE) || ($post && $post->post_type == DZSWTL_POST_TYPE)) {
      include_once DZSWTL_PATH . '/inc/php/layout-builder--functions.php';
      dzswtl_generate_layout_builder();
    }
  }

  function handle_admin_menu() {


    global $current_user;


    $admin_cap = $this->dzswtl->capability_admin;



    add_menu_page(esc_html__('Shop Builder', DZSWTL_ID), esc_html__('Shop Builder', DZSWTL_ID), $admin_cap, DZSWTL_ADMIN_PAGENAME_MAINOPTIONS, array($this, 'admin_page_mainoptions'), 'dashicons-cart');


  }

  function admin_page_mainoptions() {
    include_once DZSWTL_PATH . "class_parts/admin-page-mainoptions.php";
  }

  function handle_save_post($post_id) {


    $po = get_post($post_id);


    if ($po->post_status == 'publish') {
      $po->post_status = 'draft';
    }

    if ($po->post_title == 'Auto Draft') {
      $po->post_content = __("This is just a preview page");
    }


    if ($po->post_status != 'auto-draft' || (($po->post_parent) && $po->post_parent != '0')) {
    }


    $taxonomies = get_taxonomies('', 'names');

    foreach ($taxonomies as $taxonomy) {
      if (strpos($taxonomy, DZSWTL_QUERY_PRODUCT_ATTRIBUTE_ABBREVIATION) === 0) {

        $po_terms = wp_get_post_terms($po->ID, $taxonomy);

        if (is_array($po_terms) && count($po_terms)) {
          update_post_meta($po->ID, 'metapa_' . $taxonomy, $po_terms[0]->name);
        }
      }
    }

  }

  function handle_edit_post($post_id) {

    global $post;

    if (is_admin() && current_user_can('manage_options') == false) {
    }


    if (!$post) {
      return;
    }
    /* Check autosave */
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
      return $post_id;
    }
    if (isset($_REQUEST['dzs_nonce'])) {
      $nonce = $_REQUEST['dzs_nonce'];
      if (!wp_verify_nonce($nonce, 'dzs_nonce'))
        wp_die('Security check');
    }
    if (is_array($_POST)) {
      $auxa = $_POST;
      foreach ($auxa as $label => $value) {

        if (strpos($label, 'dzswtl_') !== false) {
          dzs_savemeta($post->ID, $label, $value);
        }
      }
    }
  }


}