<?php


class DZSWTLFilters {

  /**  @var DZSWooTable */
  public $dzswtl;


  /**
   * no pagination
   * @param $originalQuery
   * @return array
   */
  static function getCountAllQueryResults($originalQuery) {

    global $wpdb;
    $getAllQueriedIds = preg_replace('/LIMIT \d*,.*\d*?/', '', $originalQuery);
    $getAllQueriedIds = preg_replace('/LIMIT \d*?, .*?\d*?$/', 'LIMIT 0, 100000', $originalQuery);
    $results = $wpdb->get_results($getAllQueriedIds);

    return $results;
  }

  /**
   * @param $filterLabel
   * @return string
   */
  static function convertFilterLabelWithMetaTaxValueAbbrevation($filterLabel) {
    if (strpos($filterLabel, DZSWTL_QUERY_META_VALUE_ABBREVIATION) === 0) {
      $filterLabel = str_replace(DZSWTL_QUERY_META_VALUE_ABBREVIATION, '', $filterLabel);
      if (strpos($filterLabel, DZSWTL_DB_TAX_TO_META_ABBREVIATION) === 0) {
        $filterLabel = str_replace(DZSWTL_DB_TAX_TO_META_ABBREVIATION, '', $filterLabel);
        if (strpos($filterLabel, DZSWTL_QUERY_PRODUCT_ATTRIBUTE_ABBREVIATION) === 0) {
          $paTaxonomy = get_taxonomy($filterLabel);

          if ($paTaxonomy) {
            if ($paTaxonomy->label) {
              $filterLabel = $paTaxonomy->label;
            }
          }

        }
      }
    }

    if($filterLabel===DZSWTL_WOOCOMMERCE_PRODUCTS_TAG){
      $filterLabel = esc_html__("Product tag", DZSWTL_ID);
    }

    return $filterLabel;

  }

  /**
   * DZSWTLFilters constructor.
   * @param DZSWooTable $dzswtl
   */
  function __construct($dzswtl) {
    $this->dzswtl = $dzswtl;

    add_action('init', array($this, 'action_init'));

  }

  function action_init() {

    add_shortcode('dzs_woo_filter_count', array($this, 'shortcode_woo_filter_count'));
    add_shortcode('dzs_woo_filter', array($this, 'shortcode_woo_filter'));
    add_shortcode('dzs_woo_table_sorter', array($this, 'shortcode_woo_sorter'));
    add_shortcode('dzs_woo_filter_con', array($this, 'shortcode_woo_filter_con'));
    add_shortcode('dzs_woo_searcher', array($this, 'shortcode_woo_searcher'));
    add_shortcode('dzs_woo_filters_external_con', array($this, 'shortcode_filters_external_con'));

    // -- deprecated
    add_shortcode('dzs_woo_table_searcher', array($this, 'shortcode_woo_searcher'));
  }

  /**
   * [dzs_woo_filters_external_con]
   */
  function shortcode_filters_external_con($atts = array()) {

    if (!is_array($atts)) {
      $atts = array();
    }
    $shortcodeAtts = array_merge(array(
      'view_max_height' => 35
    ), $atts);
    $fout = '';

    $fout = '<div class="dzs-select--opener-external-area" data-view-max-height="' . $shortcodeAtts['view_max_height'] . '" style="">
              <div class="external-content"></div>
              <div class="external-content--show-more">
                <div class="external-content--show-more--extended">' . esc_html__('Show more', DZSWTL_ID) . '</div>
                <div class="external-content--show-more--not-extended">' . esc_html__('Show less', DZSWTL_ID) . '</div>
              </div>
            </div>';

    wp_enqueue_script('dzssel-external-area', DZSWTL_URL . 'libs/dzsselector/parts/external-opener/external-opener.js');
    wp_enqueue_style('dzssel-external-area-style', DZSWTL_URL . 'libs/dzsselector/parts/external-opener/external-opener.css');

    return $fout;
  }


  function view_generateFiltersConForWooTable($filterByList) {

    $dzswtl = $this->dzswtl;

    $fout = '';

    $fout .= '<div class="filtered-by-con ';
    if (count($filterByList) == 0) {
      $fout .= ' no-filters';
    }
    $fout .= '">';


    $foutFiltersBy = '';

    foreach ($filterByList as $filt) {

      if (isset($filt['label']) && $filt['label'] === 'sort_by') {
        continue;
      }
      $filterLabel = '';
      $filterValueDisplay = '';
      if (isset($filt['value']) && $filt['value']) {
        $filterLabel = $filt['label'];

        if (is_string($filterLabel) && is_string($filt['value'])) {
          $filterLabel = str_replace('[]', '', $filterLabel);

          $filterLabel = DZSWTLFilters::convertFilterLabelWithMetaTaxValueAbbrevation($filterLabel);


          $filterValueDisplay = urldecode($filt['value']);

          if (strpos($filterValueDisplay, DZSWTL_QUERY_WITHIN_ABBREVIATION) != false) {
            $filterValueDisplay = str_replace(DZSWTL_QUERY_WITHIN_ABBREVIATION, ' &harr; ', $filterValueDisplay);
          }
        }

        $foutFiltersBy .= '<span class="filtered-by-filter" data-lab_full="' . $filt['lab_full'] . '"><span class="the-label the-label--filtered-by--label">' . $filterLabel . '</span><span class="the-label the-label--filtered-by--value">' . $filterValueDisplay . '</span>';

        $foutFiltersBy .= '<span class="the-close delete-filter"  data-lab_full="' . $filt['lab_full'] . '">' . '<i class="fa fa-window-close"></i>' . '</span>';
        $foutFiltersBy .= '</span>';
      }

    }


    if ($foutFiltersBy) {
      $fout .= '<span class="filtered-by-label">' . esc_html__("Filtering by", DZSWTL_ID) . '</span>';
      $fout .= $foutFiltersBy;
      $fout .= '<span class="clear-all-filters dzs-button   color-normal-highlight color-over-dark"><span class="the-text"><i class="fa fa-window-close"></i> ' . esc_html__("Clear filters", DZSWTL_ID) . '</span></span>';
    }


    $fout .= '</div>';


    DZSWTLHelpers::enqueue_script('fontawesome');

    return $fout;
  }

  /**
   * @param array $pargs
   * @param string $content
   * @return string
   */
  function shortcode_woo_filter_count($pargs = array(), $content = '') {

    $shortcodeAtts = array_merge(array(

      'label' => esc_html__("Products", DZSWTL_ID), // -- not used
    ), $pargs);
    $fout = '';

    $fout .= '<div class="dzswtl-filter--count">';

    $fout .= '<span class="dzswtl-filter--count--label">' . $shortcodeAtts['label'];
    $fout .= '</span>';

    $fout .= '<span class="dzswtl-filter--count--placeholder">';
    $fout .= '</span>';
    $fout .= '</div>';

    return $fout;
  }

  /**
   * @param array $pargs
   * @param string $content
   * @return string
   */
  function shortcode_woo_filter($pargs = array(), $content = '') {

    // todo: refactor generate style
    $dzswtl = $this->dzswtl;

    $fout = '';


    include_once(DZSWTL_PATH . "class_parts/category-helpers.php");


    $filterShortcodeAtts = array(

      'title' => 'filter', // -- not used
      'feed_from' => 'categories', // -- categories or price or ratings or custom-attribute or custom-taxonomy or custom-meta
      'type' => 'detect', // slider or select or multiple or input ?
      'slider_min' => '0',
      'slider_max' => '300',
      'slider_step' => '1',
      'custom_meta_values' => '',
      'categories_type' => '',
      'show_curr_selected' => 'off', // -- show the current selected filter


      'left_label' => '',
      'parent' => '',
      'called_from' => 'default',


      'label_all' => esc_html__("All", 'dzswtl'),
      'label_auto_empty' => esc_html__("Select first", 'dzswtl'),


      'custom_taxonomy_name' => "",
      'term_parent' => "",

      'flex_align' => "",

      'flex_prop' => "",
      'margin_left' => "",
      'margin_right' => "",
      'extra_classes' => "",
      'filter_skin' => "",

      'include_count' => "off", // -- "on" or "off"
      'type_select_style' => "default", // -- default or coronae
      'type_multiple_style' => "default", // -- DZSWTL_FILTERS_TYPE_MULTIPLE_STYLES
      'type_multiple_cascade' => "off", // -- only works with type_multiple_style default for now

      'the_name' => "", // -- custom name, will overwrite auto name
    );

    if (isset($pargs) && is_array($pargs) === false) {
      $pargs = array();
    }
    $filterShortcodeAtts = array_merge($filterShortcodeAtts, $pargs);


    $isLeftLabelSet = false;

    $index_filter = $dzswtl->index_filter;


    $filterOptions = array();

    $labelFilterName = 'default';
    $labelFilterNameSane = 'default';
    $filterValue = '';

    if ($filterShortcodeAtts['the_name']) {
      if ($labelFilterName == 'default') {
        $labelFilterName = $filterShortcodeAtts['the_name'];
      }
      if ($labelFilterNameSane == 'default') {
        $labelFilterNameSane = $filterShortcodeAtts['the_name'];
      }
    }

    if ($filterShortcodeAtts['feed_from'] == 'categories') {
      // -- we will force type to multiple for now
      if ($labelFilterName == 'default') {
        $labelFilterName = 'cat';
      }
      if ($labelFilterNameSane == 'default') {
        $labelFilterNameSane = 'cat';
      }

      $taxonomyName = DZSWTL_WOOCOMMERCE_PRODUCTS_CATEGORY;


      $args_for_query = array(
        'orderby' => 'name',
        'order' => 'ASC',
        'taxonomy' => $taxonomyName,
      );

      // -- make it show only the top categories

      if ($filterShortcodeAtts['term_parent'] !== '') {
        if ($filterShortcodeAtts['term_parent']) {
          $args_for_query['parent'] = DZSWTLHelpers::sanitize_term_slug_to_id($filterShortcodeAtts['term_parent'], DZSWTL_WOOCOMMERCE_PRODUCTS_CATEGORY);

        }
      }


      // -- auto will feed from category
      if ($filterShortcodeAtts['term_parent'] === 'auto') {

        if ($labelFilterName == 'cat[]') {
          $labelFilterName = 'subcat[]';
        }
        if ($labelFilterNameSane == 'cat' || $labelFilterNameSane == 'default') {
          $labelFilterNameSane = 'subcat';
        }

        $filterShortcodeAtts['original_term_parent'] = $filterShortcodeAtts['term_parent'];

        if (isset($_GET['cat']) && $_GET['cat']) {
          $args_for_query['parent'] = DZSWTLHelpers::sanitize_term_slug_to_id($_GET['cat'], DZSWTL_WOOCOMMERCE_PRODUCTS_CATEGORY);
        }

      }

      if ($filterShortcodeAtts['type_multiple_cascade'] == 'on') {
        $args_for_query['parent'] = '0';
      }


      $catTerms = get_categories($args_for_query);


      if ($filterShortcodeAtts['type_multiple_cascade'] == 'on') {
        foreach ($catTerms as $labloop => $cat) {

          $args_for_query['parent'] = $cat->term_id;
          $subcategories = get_categories($args_for_query);

          if (count($subcategories)) {
            $catTerms[$labloop]->subcategories = $subcategories;
          }

        }
      }


      if ($filterShortcodeAtts['categories_type'] == 'category-thumb') {

        $filterShortcodeAtts['type'] = 'single-with-image';


      }


      $filterOptions = dzswtl_generate_cats_array_with_count($catTerms, $labelFilterNameSane, array(
        'type_multiple_cascade' => $filterShortcodeAtts['type_multiple_cascade']
      ));
    }


    if ($filterShortcodeAtts['feed_from'] == 'price') {

      $filterShortcodeAtts['type'] = 'slider';

      if ($labelFilterName == 'default') {

        $labelFilterName = 'price';
      }
      $labelFilterNameSane = $labelFilterName;


    }


// -- generate


    if ($filterShortcodeAtts['feed_from'] == 'ratings') {

      $filterShortcodeAtts['type'] = 'none';

      if ($labelFilterName == 'default') {
        $labelFilterName = 'rating';
      }
      $labelFilterNameSane = $labelFilterName;


      DZSWTLHelpers::enqueue_dzsSelector();


    }

    $attribute_type = '';


    // -- filter custom attribute ( DZSWTL_QUERY_PRODUCT_ATTRIBUTE_ABBREVIATION )
    if ($filterShortcodeAtts['feed_from'] == 'custom-attribute') {


      $customAttributeName = '';

      if (isset($filterShortcodeAtts['custom_attr_name'])) {
        $customAttributeName = $filterShortcodeAtts['custom_attr_name'];
      }

      $paDetails = DZSWTLHelpers::woocommerceGetDetailsForProductAttribute($customAttributeName, $filterShortcodeAtts, $labelFilterNameSane);

      $labelFilterName = $paDetails['labelFilter'];
      $labelFilterNameSane = $paDetails['labelFilterSane'];

      if (isset($dzswtl->sanitized_get) && is_array($dzswtl->sanitized_get) && count($dzswtl->sanitized_get) > 0) {
        if (isset($dzswtl->sanitized_get[$labelFilterNameSane])) {
          $cats = $dzswtl->sanitized_get[$labelFilterNameSane];
          if (strpos($dzswtl->sanitized_get[$labelFilterNameSane], '%252C') !== false) {
            $cats = str_replace('%252C', ',', $dzswtl->sanitized_get[$labelFilterNameSane]);
          }
          if (strpos($dzswtl->sanitized_get[$labelFilterNameSane], '%2C') !== false) {
            $cats = str_replace('%2C', ',', $dzswtl->sanitized_get[$labelFilterNameSane]);
          }
          $cats = explode(',', $cats);
          $filterValue = $cats;
        }
      }

      $filterOptions = $paDetails['terms_processed'];

    }


    if ($filterShortcodeAtts['feed_from'] == 'custom-taxonomy') {


      $taxonomyName = $filterShortcodeAtts['custom_taxonomy_name'];
      $labelFilterNameSane = DZSWTL_QUERY_CUSTOM_TAX_ABBREVIATION . $filterShortcodeAtts['custom_taxonomy_name'];
      $labelFilterName = DZSWTL_QUERY_CUSTOM_TAX_ABBREVIATION . $filterShortcodeAtts['custom_taxonomy_name'] . '[]';


      $tax = get_taxonomy($taxonomyName);


      $catTerms = get_terms($taxonomyName, array(
        'hide_empty' => false,
      ));


      $cats = dzswtl_cats_get_from_get($labelFilterNameSane);
      $filterValue = $cats;


      // -- custom taxonomy
      foreach ($catTerms as $cat) {


        $aux = array(
          'label' => $cat->name,
          'value' => $cat->slug,
          'term_id' => $cat->term_id,
        );


        $aux['count'] = $cat->count;
        if ($attribute_type == 'color') {
          $aux['meta_color'] = get_term_meta($cat->term_id, 'color', true);
        }


        array_push($filterOptions, $aux);
      }

    }

    if ($filterShortcodeAtts['feed_from'] == 'custom-meta') {

      $customMetaName = '';

      if (isset($filterShortcodeAtts['custom_meta_name'])) {
        $customMetaName = $filterShortcodeAtts['custom_meta_name'];
      }

      if ($customMetaName === '') {
        return 'no meta found';
      }
      $customMetaName = $filterShortcodeAtts['custom_meta_name'];


      $labelFilterName = DZSWTL_QUERY_META_VALUE_ABBREVIATION . $filterShortcodeAtts['custom_meta_name'];
      $labelFilterNameSane = $labelFilterName;
      $meta_name = $filterShortcodeAtts['custom_meta_name'];


      $attr_data_str = $filterShortcodeAtts['custom_meta_values'];


      $tax = get_taxonomy($customMetaName);


      $cats = dzswtl_cats_get_from_get($labelFilterNameSane);
      $filterValue = $cats;


      $catTerms = explode(',', $filterShortcodeAtts['custom_meta_values']);


      // -- custom meta
      if (is_array($catTerms) && count($catTerms)) {

        foreach ($catTerms as $cat) {


          $aux = array(
            'label' => $cat,
            'value' => $cat,
          );

          array_push($filterOptions, $aux);
        }
      }

    }


    // -- we start to display the shortcode

    if ($filterShortcodeAtts['type'] == '' || $filterShortcodeAtts['type'] == 'detect') {
      $filterShortcodeAtts['type'] = DZSWTL_FILTERS_TYPES['MULTIPLE'];
    }

    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['SLIDER']) {


      // -- if it's slider we force product attribute to meta
      if (strpos($labelFilterName, DZSWTL_QUERY_PRODUCT_ATTRIBUTE_ABBREVIATION) === 0) {
        $labelFilterName = DZSWTL_QUERY_META_VALUE_ABBREVIATION . '' . DZSWTL_DB_TAX_TO_META_ABBREVIATION . '' . $labelFilterName;
        $labelFilterNameSane = $labelFilterName;
      }


      // -- why we need [] on slider ?
      $labelFilterName = str_replace('[]', '', $labelFilterName);
      $labelFilterNameSane = str_replace('[]', '', $labelFilterNameSane);
    }


    // -- finish labs

    /**
     * start to generate fout
     */

    // -- start to generate $fout
    $fout = '';

    $fout .= '<div class="dzs-woo-filter';


    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['SELECT']) {

      $fout .= ' select-style-' . $filterShortcodeAtts['type_select_style'];
    }
    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['MULTIPLE']) {

      $fout .= ' multiple-style-' . $filterShortcodeAtts['type_multiple_style'];
    }

    if ($filterShortcodeAtts['flex_align']) {
      $fout .= ' flex-align-' . $filterShortcodeAtts['flex_align'];
    }
    if ($filterShortcodeAtts['flex_prop']) {
      $fout .= ' flex-prop-' . $filterShortcodeAtts['flex_prop'];
    }

    if ($filterShortcodeAtts['filter_skin']) {
      $fout .= ' ' . $filterShortcodeAtts['filter_skin'];
      wp_enqueue_style('dzswoo-filter-part-skin-' . $filterShortcodeAtts['filter_skin'], DZSWTL_URL . 'libs/dzswootable/parts/filters/skins/filter-skin--' . $filterShortcodeAtts['filter_skin'] . '.css');
    }

    if ($filterShortcodeAtts['extra_classes']) {
      $fout .= ' ' . $filterShortcodeAtts['extra_classes'];
    }

    $fout .= '"';


    $fout .= ' data-lab-sanitized="' . $labelFilterNameSane . '"';
    $fout .= ' style="';
    $fout .= dzswtl_generate_style_for_flex($filterShortcodeAtts);


    $fout .= '">';


    if ($filterShortcodeAtts['feed_from'] == DZSWTL_FEED_FROM['RATINGS']) {
      $html = dzs_read_from_file_ob(DZSWTL_PATH . 'class_parts/html/ratings.html');
      $html = str_replace('{{$lab_sanitized}}', $labelFilterNameSane, $html);
      $html = str_replace('{{__AllRatings}}', esc_html__("All ratings", 'dzswtl'), $html);
      $fout .= $html;
    };


    DZSWTLHelpers::enqueue_dzsSelector();


    // -- here we have the woo-feed filter

    $fout .= '<div class="dzs-woo-feed dzs-woo-feed-filter-obj ';


    if ($filterShortcodeAtts['term_parent']) {
      $fout .= ' dzs-woo-feed-filter-from';
    }

    $fout .= '" data-dzs-woo-feed-lab="' . $labelFilterNameSane . '" data-dzs-woo-feed-type="' . $filterShortcodeAtts['type'] . '"';

    if ($filterShortcodeAtts['term_parent']) {
      $fout .= '  data-dzs-woo-term-parent="' . $filterShortcodeAtts['term_parent'].'"';
    }


    $fout .= ' ></div>';

    if ($filterShortcodeAtts['left_label'] && !$isLeftLabelSet) {

      $fout .= '<span class="woo-filter-slider--label">';
      $fout .= $filterShortcodeAtts['left_label'];
      $fout .= '</span>';

      $isLeftLabelSet = true;
    }

// -- single with image
    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['SINGLE_WITH_IMAGE']) {
      foreach ($filterOptions as $opt) {


        $classes = '';


        $fout .= '<div class="dzs-woo-table-filter-for-click';


        if (isset($_GET) && is_array($_GET) && count($_GET) > 0) {


          if (isset($_GET[$labelFilterNameSane])) {


            if ($_GET[$labelFilterNameSane] == $opt['value']) {

              $fout .= ' active';
            }
          }
        }


        $fout .= '" data-term-slug="' . $opt['value'] . '">';


        $term_id = $opt['value'];


        $term_id = '';


        if (is_numeric($opt['value'])) {
          $term_id = $opt['value'];
        } else {

          $term = get_term_by('slug', $opt['value'], DZSWTL_WOOCOMMERCE_PRODUCTS_CATEGORY);

          $term_id = $term->term_id;
        }


        $term_meta_all = get_term_meta($term_id);


        $thumb_id = get_term_meta($term_id, 'thumbnail_id', true);

        if ($thumb_id == '') {
          $thumb_id = get_term_meta($term_id, 'product_cat_thumbnail', true);
        }
        if ($thumb_id == '') {
          $thumb_id = get_term_meta($term_id, 'image', true);
        }

        if ($thumb_id) {

          $thumb_src = DZSWTLHelpers::sanitize_id_to_src($thumb_id);
          $fout .= '<span class="dzs-woo-table-filter-for-click--image divimage" style="background-image: url(' . $thumb_src . '); ">';

          $fout .= '</span>';
        }


        $fout .= '<span class="dzs-woo-table-filter-for-click--label">';
        $fout .= $opt['label'];
        $fout .= '</span>';


        $fout .= '</div>';

      }

    }


    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['MULTIPLE']) {


      // -- multiple type
      $isMultipleStyle = false;

      if (in_array($filterShortcodeAtts['type_multiple_style'], DZSWTL_FILTERS_TYPE_MULTIPLE_STYLES)) {
        $isMultipleStyle = true;
      }


      // -- checkbox search
      if ($isMultipleStyle) {
        // --- only some styles

        $isEmbeddingExternalStyle = false;
        $isDisablingSearch = false;

        if ($filterShortcodeAtts['type_multiple_style'] == DZSWTL_FILTERS_TYPE_MULTIPLE_STYLES['pill'] || $filterShortcodeAtts['type_multiple_style'] == DZSWTL_FILTERS_TYPE_MULTIPLE_STYLES['small-image-with-title']) {
          $isEmbeddingExternalStyle = true;
        }
        if ($filterShortcodeAtts['type_multiple_style'] == DZSWTL_FILTERS_TYPE_MULTIPLE_STYLES['small-checkbox'] || $filterShortcodeAtts['type_multiple_style'] == DZSWTL_FILTERS_TYPE_MULTIPLE_STYLES['small-image-with-title'] || $filterShortcodeAtts['type_multiple_style'] == DZSWTL_FILTERS_TYPE_MULTIPLE_STYLES['pill']) {
          $isDisablingSearch = true;
        }


        if ($isEmbeddingExternalStyle) {
          wp_enqueue_style('dzssel-external-area', DZSWTL_URL . 'libs/dzsselector/parts/style--opener-list--' . $filterShortcodeAtts['type_multiple_style'] . '/style--opener-list--' . $filterShortcodeAtts['type_multiple_style'] . '.css');
        }

        $fout .= '<div class="dzs-style-me type-checkbox skin-beige opener-list opener-list-absolute type-multiple-style--' . $filterShortcodeAtts['type_multiple_style'] . ' style--opener-list--' . $filterShortcodeAtts['type_multiple_style'];

        if ($isDisablingSearch) {
          $fout .= ' disable-search';
          $fout .= ' ' . $filterShortcodeAtts['type_multiple_style'];
        }

        $fout .= '">';


        // -- multiple label all
        if ($filterShortcodeAtts['show_curr_selected'] == 'on') {

          $fout .= '<div class="feed-dzswtl label--multiple label-curr-selected" hidden aria-hidden="true" style="">';

          if (isset($dzswtl->all_checks[$labelFilterNameSane])) {
            $fout .= $dzswtl->all_checks[$labelFilterNameSane];
          }

          $fout .= '</div>';
        }
        $fout .= '<div hidden class="feed-dzswtl label--multiple feed-dzswtl--placeholder label-all" style=";">' . $filterShortcodeAtts['label_all'] . '</div>';
        $fout .= '<div class="feed-dzswtl label--multiple label-auto-empty" hidden aria-hidden="true">' . $filterShortcodeAtts['label_auto_empty'] . '</div>';


        $fout .= '<div class="dzs-style-me-feeder">';
      }


      if ($attribute_type == 'color') {
        $fout .= '<div class="dzs-checkbox-selector skin-colornova">';
      }


      // -- multiple
      // -- style simple
      foreach ($filterOptions as $opt) {

        $fout .= dzswtl_view_generateMultipleOption($opt, $filterShortcodeAtts, $attribute_type, $labelFilterNameSane, $labelFilterName);
      }


      if ($attribute_type == 'color') {
        $fout .= '</div>';
      }


      if ($isMultipleStyle) {
        $fout .= '</div></div>';
      }


    }


    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['SLIDER']) {


      $input_name = $labelFilterNameSane;


      $rangeStart = $filterShortcodeAtts['slider_min'];
      $rangeEnd = $filterShortcodeAtts['slider_max'];
      $sliderStep = $filterShortcodeAtts['slider_step'];
      $initialValue = '';
      $valueStart = $rangeStart;
      $valueEnd = $rangeEnd;

      if (isset($_GET[$labelFilterNameSane]) && $_GET[$labelFilterNameSane]) {
        $price_range = explode(DZSWTL_QUERY_WITHIN_ABBREVIATION, $_GET[$labelFilterNameSane]);
        if (is_array($price_range) && count($price_range) > 1) {
          $valueStart = $price_range[0];
          $valueEnd = $price_range[1];
          $initialValue = $_GET[$labelFilterNameSane];
        }

      }


      $fout .= '<div class="woo-filter-slider-con">';

      if ($filterShortcodeAtts['left_label'] && $isLeftLabelSet == false) {

        $fout .= '<div class="woo-filter-slider--label">';
        $fout .= $filterShortcodeAtts['left_label'];
        $fout .= '</div>';

        $isLeftLabelSet = true;
      }
      $fout .= '
<input class="dzs-woo-table-filter-input for-slider" name="' . $input_name . '" type="text"  style="border:0; color:#f6931f; font-weight:bold;" value="' . $initialValue . '" />
<div class="dzs-slider-auto-init" id="slider-range' . $index_filter . '" data-slider-min="' . $rangeStart . '" data-slider-max="' . $rangeEnd . '">';

      $fout.='<div class="dzs-woo-slider-handle ui-slider-handle handle-min"></div><div class="dzs-woo-slider-handle ui-slider-handle handle-max"></div>';

      $fout.='</div>'; // -- end .dzs-slider-auto-init
      $fout.='</div>'; // -- end .woo-filter-slider-con


      include_once(DZSWTL_PATH . 'inc/php/view-functions/shortcode-woo-filter-helpers.php');


      wp_enqueue_script('jquery-ui-core');
      wp_enqueue_script('jquery-ui-slider');
      wp_enqueue_style('jquery-ui', 'https://code.jquery.com/ui/1.12.1/themes/' . $dzswtl->db_mainoptions['jquery_ui_theme'] . '/jquery-ui.css');
      wp_enqueue_script('dzs-ui-slider', DZSWTL_URL . 'libs/dzswootable/parts/filters/ui-slider/dzs-ui-slider.js');


    }


    // -- select type
    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['SELECT']) {

      // -- select type START

      $fout .= '<div class="feed-dzswtl label-all" style="display:none;">' . $filterShortcodeAtts['label_all'] . '</div>';
      $fout .= '<div class="feed-dzswtl label-auto-empty" hidden aria-hidden="true">' . $filterShortcodeAtts['label_auto_empty'] . '</div>';


      // -- prepend label all on top
      if ($filterShortcodeAtts['label_all']) {
        // todo: allow deselect and do not include this in the filters
        array_unshift($filterOptions, array(
          'label' => ($filterShortcodeAtts['label_all']),
          'value' => '',
        ));
      }

      $argsForFormGenerate = array(
        'options' => $filterOptions,
        'class' => 'dzs-woo-table-filter-input filter-type-select feed-from-' . $filterShortcodeAtts['feed_from'] . ' dzs-style-me skin-beige do-not-trigger-change-on-init do-not-trigger-change-on-reinit  ', // -- dzs-woo-table-filter-input
      );

      $labelFilterName = $labelFilterNameSane;

      if (is_array($filterValue) && count($filterValue) == 1) {
        $filterValue = $filterValue[0];
      }

      $argsForFormGenerate['seekval'] = $filterValue;


      if ($filterShortcodeAtts['type_select_style'] == 'default') {

        $fout .= DZSHelpers::generate_select($labelFilterName, $argsForFormGenerate);
      }

      if ($filterShortcodeAtts['type_select_style'] == 'coronae' || $filterShortcodeAtts['type_select_style'] == 'forseen' || $filterShortcodeAtts['type_select_style'] == 'tabs-menu') {


        $argsForFormGenerate['class'] = 'dzs-style-me opener-listbuttons dzs-woo-table-filter-input filter-type-select feed-from-' . $filterShortcodeAtts['feed_from'] . ' do-not-trigger-change-on-init do-not-trigger-change-on-reinit ';
        $fout .= DZSHelpers::generate_select($labelFilterName, $argsForFormGenerate);


        $fout .= '<ul class="dzs-style-me-feeder">';


        foreach ($argsForFormGenerate['options'] as $labelFilterName => $val) {

          $fout .= ' <li ><span class="">' . $val['label'] . '</span></li>';
        }

        $fout .= '</ul>';
      }


    }


// -- select type
    if ($filterShortcodeAtts['type'] == DZSWTL_FILTERS_TYPES['INPUT']) {


      // -- prepend label all on top
      if ($filterShortcodeAtts['label_all']) {

        array_unshift($filterOptions, array(
          'label' => ($filterShortcodeAtts['label_all']),
          'value' => '',
        ));
      }


      $class = 'dzs-woo-table-filter-input filter-type-input feed-from-' . $filterShortcodeAtts['feed_from'] . ' ';
      $labelFilterName = $labelFilterNameSane;


      if (is_array($filterValue) && count($filterValue) == 1) {
        $filterValue = $filterValue[0];
      }

      $argsForFormGenerate['seekval'] = $filterValue;
      $argsForFormGenerate['class'] = $class;


      $argsForFormGenerate['extraattr'] = ' placeholder="' . $filterShortcodeAtts['label_auto_empty'] . '"';

      $fout .= DZSHelpers::generate_input_text($labelFilterName, $argsForFormGenerate);
    }

    if ($filterShortcodeAtts['type'] != DZSWTL_FILTERS_TYPES['SINGLE_WITH_IMAGE']) {
    }
    $fout .= '</div><!-- end .dzs-woo-filter -->';

    $dzswtl->index_filter++;

    if ($filterShortcodeAtts['called_from'] !== 'dzswtl_widget') {

    }
    DZSWTLHelpers::enqueue_script();
    DZSWTLHelpers::enqueue_script('filters');


    return $fout;


  }

  function shortcode_woo_sorter($pargs = array()) {

    global $dzswtl;


    $fout = '';


    include_once(DZSWTL_PATH . "class_parts/sorter.php");
    $fout .= dzswtl_view_sorter($pargs);
    return $fout;
  }

  /**
   * [dzs_woo_filter_con extra_classes=""]
   * @param array $pargs
   * @param string $content
   * @return string
   */
  function shortcode_woo_filter_con($pargs = array(), $content = '') {

    global $dzswtl;


    $margs = array(

      'extra_classes' => ' ',
    );


    if (is_array($pargs)) {

      $margs = array_merge($margs, $pargs);
    }


    $fout = '';


    $fout .= '<div class="woo-filter-con ' . $margs['extra_classes'] . '">';
    $fout .= do_shortcode($content);
    $fout .= '</div><!-- end woo-filter-con-->';


    wp_enqueue_style('dzswoo-filter-con', DZSWTL_URL . 'libs/dzswootable/parts/filters/filters-con/filters-con.css');
    return $fout;


  }


  /**
   * [dzs_woo_searcher]
   * @param array $pargs
   * @return string
   */
  function shortcode_woo_searcher($pargs = array()) {

    global $dzswtl;


    $margs = array(

      'style_float' => '',
      'flex_align' => '',
      'flex_prop' => '',
      'link' => '', // -- it can link to another page
      'class' => '', // -- search-style-baroque

    );

    // -- here is search


    if (is_array($pargs)) {

      $margs = array_merge($margs, $pargs);
    }


    $fout = '';

    $lab = 'woo_search';


    $fout .= '<div class="woo-table-searcher ' . $margs['class'] . '" style="';


    if ($margs['style_float']) {
      $fout .= ' float: ' . $margs['style_float'] . ';';
    }


    $fout .= dzswtl_generate_style_for_flex($margs);

    $fout .= '">';


    if ($margs['link']) {
      $fout .= '<form action="' . $margs['link'] . '" method="GET">';
      $lab = 'search';
    }

    $val = '';


    if (isset($_GET['search'])) {
      $val = $_GET['search'];
    } else {
      if (isset($_GET['the_search'])) {
        $val = $_GET['the_search'];
      }
    }


    $fout .= DZSHelpers::generate_input_text($lab, array(
      'class' => 'dzs-field ',
      'seekval' => $val,
      'extraattr' => ' placeholder="' . __("Search") . '... "',

    ));
    if ($margs['link']) {
      $fout .= '</form>';
    }

    $fout .= '</div>';

    DZSWTLHelpers::enqueue_script('filters');

    return $fout;
  }


}