<?php


class DZSWTLHelpers {


  static function parseItem_generatePriorityOrders($priorityIndex, $priorityIsAutoHide, $priority_blocks) {
    $fout = '';
    if ($priorityIndex !== null) {
      $fout .= ' data-priority-index="' . $priorityIndex . '"';

      if (intval($priorityIndex) < ceil(count($priority_blocks) / 2)) {
        $fout .= ' data-priority-is-important-block="on"';
      }
    }
    if ($priorityIsAutoHide !== null && $priorityIsAutoHide === 'on') {
      $fout .= ' data-priority-allow-hide="' . 'on' . '"';
    }

    return $fout;
  }

  /**
   * @param string $arg -- slug
   * @param string $taxonomy_name
   * @return int|mixed|string
   */
  static function sanitize_term_slug_to_id($arg, $taxonomy_name = '') {


    $taxonomyName = DZSWTL_WOOCOMMERCE_PRODUCTS_CATEGORY;

    if ($taxonomy_name) {
      $taxonomyName = $taxonomy_name;
    }


    if (is_numeric($arg)) {

    } else {

      $term = get_term_by('slug', $arg, $taxonomyName);


      if ($term) {
        $arg = $term->term_id;
      }

    }


    return $arg;
  }


  public static function user_has_role_cap($u, $role_or_cap) {


    //$u->roles Wrong way to do it as in the accepted answer.
    $roles_and_caps = $u->get_role_caps(); //Correct way to do it as wp do multiple checks to fetch all roles

    if (isset ($roles_and_caps[$role_or_cap]) and $roles_and_caps[$role_or_cap] === true) {
      return true;
    }

  }

  static function db_addUserDownloadedTrack($trackId) {

    $downloadedTracks = array();

    $currentUserId = get_current_user_id();

    if ($currentUserId) {

      $downloadedTracksString = get_user_meta($currentUserId, DZSPSY_DB_USER_META_DOWNLOADED_TRACKS, true);

      if ($downloadedTracksString) {
        $downloadedTracks = json_decode($downloadedTracksString);
      }

      if (!$downloadedTracks) {
        $downloadedTracks = array();
      }

      if (!in_array($trackId, $downloadedTracks)) {
        array_push($downloadedTracks, $trackId);
        update_user_meta($currentUserId, DZSPSY_DB_USER_META_DOWNLOADED_TRACKS, json_encode($downloadedTracks));
      }

    }
  }

  /**
   * DZSPSY is defined
   * @param int $deductable_points
   * @param array $pargs
   * @return array
   */
  public static function view_canUserDownload($deductable_points, $pargs = array()) {
    global $woocommerce, $dzswtl, $dzspsy;

    $margs = array(
      'subtotal' => ''
    );

    $margs = array_merge($margs, $pargs);

    $cart = $woocommerce->cart;

    $respArr = array(
      'can_download' => true,
      'can_download_status' => '',
      'is_user_already_downloaded' => false,
      'deductable_points' => $deductable_points,
    );
    if ($margs['subtotal']) {
      $subtotal = $margs['subtotal'];
    } else {
      $subtotal = ceil($cart->get_subtotal());
    }
    if ($subtotal < $deductable_points) {
      $deductable_points = $subtotal;

      $respArr['can_download'] = false;
      $respArr['can_download_status'] = 'subtotal-lower-then-points';
    }
    $user_points = 0;
    if ($dzspsy) {
      $user_points = $dzspsy->classDb->get_points();
    }
    if ($user_points < $deductable_points) {
      $deductable_points = $user_points;
      $respArr['can_download'] = false;
      $respArr['can_download_status'] = 'user-points-lower-then-points';
    }

    $itemsInCart = $cart->get_cart_contents();
    foreach ($itemsInCart as $cartItem) {

      if ($cartItem['product_id']) {
        if (defined('DZSPSY_WOOCOMMERCE_PRODUCTS_CATEGORIES')) {

          $cats = get_the_terms($cartItem['product_id'], DZSPSY_WOOCOMMERCE_PRODUCTS_CATEGORIES);

          if ($dzswtl->db_mainoptions['excluded_categories'] && is_array($dzswtl->db_mainoptions['excluded_categories'])) {
            foreach ($cats as $cat) {
              if (in_array($cat->term_id, $dzswtl->db_mainoptions['excluded_categories'])) {

                $respArr['can_download'] = false;
                $respArr['can_download_status'] = 'you-have-products-that-cannot-be-purchased-with-points';
              }

            }

          }
        }
      }
    }
    $respArr['deductable_points'] = $deductable_points;
    return $respArr;
  }


  static function view_isProductDownloaded($postId) {

    $currentUserId = get_current_user_id();

    if ($currentUserId) {


      $downloadedTracks = array();
      $downloadedTracksString = get_user_meta($currentUserId, DZSPSY_DB_USER_META_DOWNLOADED_TRACKS, true);
      if ($downloadedTracksString) {
        $downloadedTracks = json_decode($downloadedTracksString);
      }
      if (in_array(intval($postId), $downloadedTracks) || in_array(strval($postId), $downloadedTracks)) {
        return true;
      }


      $customer_orders = wc_get_orders($args = array(
        'limit' => -1,
        'customer_id' => $currentUserId, // The current user id
        'status' => array_keys(wc_get_order_statuses()),
      ));

// The different loops to get the downloadable products bought by this user
      if (!empty($customer_orders)) {
        // Loop through customer orders
        foreach ($customer_orders as $order) {
          // Check if current order has available downloadable items (permitted)
          if ($order->has_downloadable_item() && $order->is_paid() && $order->is_download_permitted()) {
            // Loop through order items with downloadable items
            foreach ($order->get_items() as $item) {
              $product_id = $item->get_product_id(); // product ID

            }
          }
        }
      }

    }

    return false;
  }

  public static function get_wishlist($pargs = array()) {


    $margs = array(
      'meta_name' => DZSWTL_POST_META_WISHLIST_NAME,
      'call_from' => 'default'
    );


    if (is_array($pargs)) {
      $margs = array_merge($margs, $pargs);
    }


    $arr_wishlist = array();


    $arr_wishlist_str = '';
    if (get_current_user_id()) {

      if (get_user_meta(get_current_user_id(), $margs['meta_name'], true) && get_user_meta(get_current_user_id(), $margs['meta_name'], true) != 'null') {

        $arr_wishlist_str = get_user_meta(get_current_user_id(), $margs['meta_name'], true);
      }
    } else {


      if (isset($_COOKIE[$margs['meta_name']])) {
        $arr_wishlist_str = stripslashes($_COOKIE[$margs['meta_name']]);
      }
    }


    if ($arr_wishlist_str) {

      try {

        $arr_wishlist = json_decode($arr_wishlist_str, true);
      } catch (Exception $e) {

      }
    }

    if ($margs['call_from'] != 'do_not_repeat' && count($arr_wishlist) == 0) {
      $arr_wishlist = DZSWTLHelpers::get_wishlist(array(
        'meta_name' => 'dzsap_wishlist',
        'call_from' => 'do_not_repeat'
      ));
    }


    return $arr_wishlist;
  }


  public static function productIsInWishlist($postObject) {

    $arr_wishlist = DZSWTLHelpers::get_wishlist();
    return !!(in_array($postObject->ID, $arr_wishlist));

  }

  /**
   * only if currecnt user can download
   * @param WP_Post $po
   * @param bool $isForView - for viewing the download button, else only for permission sake
   * @return bool - can download or not / isPurchased / isAdmin
   */
  public static function productIsDownloadable($po, $isForView = false) {
    global $dzswtl;

    $loggedInUserId = get_current_user_id();

    $isCanDownload = false;

    if ($dzswtl->db_mainoptions['downloads_allow_with_only_login'] === 'on') {
      if ($loggedInUserId == 0) {
        return false;
      }
      if ($dzswtl->db_mainoptions['allow_download_only_for_registered_users_capability'] && $dzswtl->db_mainoptions['allow_download_only_for_registered_users_capability'] != 'read') {
        $u = new WP_User($loggedInUserId);
        if (!(current_user_can(DZSWTL_PERMISSION_ULTIMATE) || DZSWTLHelpers::user_has_role_cap($u, $dzswtl->db_mainoptions['allow_download_only_for_registered_users_capability']))) {

          return false;
        }
      }
    }
    if (defined('DZSAP_VERSION')) {
      if ($po->post_type == DZSAP_REGISTER_POST_TYPE_NAME) {
        $isCanDownload = true;
      }
    }
    if (!$isForView && current_user_can(DZSWTL_PERMISSION_ULTIMATE)) {
      $isCanDownload = true;
    }


    if ($po->post_type == 'product') {

      // Get all customer orders
      $customer_orders = get_posts(array(
        'numberposts' => -1,
        'meta_key' => '_customer_user',
        'meta_value' => $loggedInUserId,
        'post_type' => 'shop_order', // WC orders post type
        'post_status' => 'wc-completed' // Only orders with status "completed"
      ));
      foreach ($customer_orders as $customer_order) {
        // -- Updated compatibility with WooCommerce 3+

        if (function_exists('wc_get_order')) {

          $order = wc_get_order($customer_order);

          // Iterating through each current customer products bought in the order
          foreach ($order->get_items() as $item) {
            $product_id = $item->get_product_id();

            if ($product_id === $po->ID)
              $isCanDownload = true;
          }
        }
      }


      $price = null;

      if (function_exists('wc_get_product')) {

        $product = wc_get_product($po->ID);


        if ($dzswtl->db_mainoptions['downloads_allow_with_price_zero'] === 'on') {
          if ($product) {
            $price = $product->get_price();
          }
          if ($price !== null && (intval($price) === 0 || $price === '0')) {
            $isCanDownload = true;
          }
        }
        if (!$product->is_downloadable()) {
          $isCanDownload = false;
        }
      }
    }


    $isCanDownload = apply_filters(DZSWTL_FILTER_NAME__IS_CAN_DOWNLOAD, $isCanDownload, $po);
    return $isCanDownload;
  }

  public static function productGetDownloadUrl($po) {
    global $dzsap;

    $downloadFileUrl = '';
    $downloadFilePath = '';

    if (defined('DZSAP_VERSION')) {
      if ($po->post_type == DZSAP_REGISTER_POST_TYPE_NAME) {
        $ceva = 0;
        $cevamargs = array();
        $downloadFileUrl = $dzsap->get_track_source($po->ID, $ceva, $cevamargs);

        $downloadFilePath = '';

        if (strpos($downloadFileUrl, site_url()) !== false) {
          $downloadFilePath = str_replace(site_url(), ABSPATH, $downloadFileUrl);
        }

      }
    }


    if ($downloadFileUrl == '') {
      if (function_exists('get_field')) {
        $arr = get_field('scratch_preview', $po->ID);
        if ($arr) {
          $media = wp_get_attachment_url($arr);
          $downloadFileUrl = $media;
        }
      }
    }


    if ($po->post_type == 'product') {
      if (get_post_meta($po->ID, 'dzsap_woo_use_preview_track_for_download', true) == 'yes') {
        if (get_post_meta($po->ID, 'dzsap_woo_product_track', true)) {
          $downloadFileUrl = get_post_meta($po->ID, 'dzsap_woo_product_track', true);
        }
      } else {
        if (class_exists('WC_Product_Factory')) {
          $productObject = new WC_Product_Factory();
          $product = $productObject->get_product($po->ID);


          if ($product->is_downloadable()) {

            $downloads = $product->get_downloads();

            foreach ($downloads as $key => $each_download) {
              $downloadFileUrl = $each_download['file'];

            }
          }
          if (!$downloadFileUrl && defined('DZSAP_VERSION')) {
            if (get_post_meta($po->ID, 'dzsap_woo_product_track', true)) {
              $downloadFileUrl = get_post_meta($po->ID, 'dzsap_woo_product_track', true);
            }
          }
        }

      }
    }


    if (strpos($downloadFileUrl, site_url()) !== false) {
      $aux = str_replace(site_url() . '/', ABSPATH, $downloadFileUrl);
      if (file_exists($aux)) {
        $downloadFilePath = $aux;
      }
    }

    if ($downloadFilePath) {
      return $downloadFilePath;
    }
    return $downloadFileUrl;

  }

  public static function init_sanitizeGetArray($dzswtl) {
    // -- let us sanitize get input

    $dzswtl->sanitized_get = array_merge(array(), $_GET);

    foreach ($dzswtl->sanitized_get as $lab => $val) {
      if (strpos($val, DZSWTL_QUERY_WITHIN_ABBREVIATION) !== false) {
        $arr = explode(DZSWTL_QUERY_WITHIN_ABBREVIATION, $val);
        if (isset($dzswtl->sanitized_get[$lab . DZSWTL_QUERY_MINIMUM_ABBREVIATION]) == false) {
          $dzswtl->sanitized_get[$lab . DZSWTL_QUERY_MINIMUM_ABBREVIATION] = $arr[0];
        }
        if (isset($dzswtl->sanitized_get[$lab . DZSWTL_QUERY_MAXIMUM_ABBREVIATION]) == false) {
          if ($arr[1] == 'infinite') {
            $arr[1] = '99999999';
          }
          $dzswtl->sanitized_get[$lab . DZSWTL_QUERY_MAXIMUM_ABBREVIATION] = $arr[1];
        }
      }
    }
  }

  public static function checkIfItShouldInvalidateAnchorUpperOfForm($ld) {

    if (isset($ld['button_type']) && $ld['button_type'] == 'add-to-cart' && isset($ld['button_functionality']) && $ld['button_functionality'] == 'default' && (isset($ld['button_style']) && $ld['button_style'] == 'default')) {
      return true;
    }
    return false;
  }

  public static function get_product_images($pid, $w = 300) {

    global $product;


    $arr = array();


    $product = null;
    if (function_exists('wc_get_product')) {

      $product = wc_get_product($pid);

      if ($product) {


        $attachment_ids = $product->get_gallery_image_ids();

        foreach ($attachment_ids as $attachment_id) {


          $im_sizes = array(
            $w, 0
          );


          $img_arr = wp_get_attachment_image_src($attachment_id, $im_sizes);


          $img_src = '';

          if (isset($img_arr[0])) {

            $img_src = $img_arr[0];
          }

          $aux = array(
            'id' => $attachment_id,
            'url' => $img_src,
          );


          array_push($arr, $aux);
        }
      }
    }


    if (count($arr) == 0) {
      $arr = array(array(
        'id' => '0',
        'url' => DZSWTLHelpers::get_thumbnail_src(get_post($pid)),
      )
      );
    }

    return $arr;
  }


  /**
   * @param WC_Product $product
   * @return bool
   */
  public static function woocommerceProductIsOutOfStock($product) {


    return ($product->managing_stock() && !$product->is_in_stock());
  }

  /**
   * @param $taxonomyName
   * @param array $margs
   * @param string $lab_sanitized
   * @return array [
   *     'labelFilterSane' => '',
   *     'labelFilter' => '',
   *     'attribute_type' => '',
   *     'terms' => [],
   *     'terms_processed' => [],
   * ]
   */
  public static function woocommerceGetDetailsForProductAttribute($taxonomyName, $margs = array(), $lab_sanitized = 'default') {
    $paDetails = array();


    if ($lab_sanitized == 'default') {
      $lab_sanitized = $taxonomyName;
    }
    $lab = $taxonomyName . '[]';
    $tax = get_taxonomy($taxonomyName);
    $attribute_taxonomies = null;

    if (function_exists('wc_get_attribute_taxonomies')) {
      $attribute_taxonomies = wc_get_attribute_taxonomies();
    }

    $attribute_type = 'text';

    if ($attribute_taxonomies) {
      foreach ($attribute_taxonomies as $atax) {
        if (DZSWTL_QUERY_PRODUCT_ATTRIBUTE_ABBREVIATION . $atax->attribute_name == $lab_sanitized) {
          $attribute_type = $atax->attribute_type;
        }
      }
    }


    $args = array(
      'hide_empty' => false,
    );


    if (isset($margs['parent']) && $margs['parent']) {
      $args['parent'] = $margs['parent'];
    }


    $terms = get_terms($taxonomyName, $args);


    $termsProcessed = array();

    // -- custom taxonomy
    foreach ($terms as $cat) {
      // -- custom attributes


      if (!isset($cat->name)) {
        continue;
      }
      $aux = array(
        'label' => $cat->name,
        'value' => $cat->slug,
        'term_id' => $cat->term_id,
      );


      $aux['count'] = $cat->count;


      if ($attribute_type == 'color') {

        $aux['meta_color'] = get_term_meta($cat->term_id, 'color', true);
      }


      array_push($termsProcessed, $aux);
    }


    return array(
      'labelFilter' => $lab,
      'labelFilterSane' => $lab_sanitized,
      'terms' => $terms,
      'attribute_type' => $attribute_type,
      'terms_processed' => $termsProcessed,
    );

  }

  /**
   * dzswtl initial setup
   * @param $dzswtl
   */
  public static function mainInitialSetup($dzswtl) {


    if (!is_admin()) {

      if ($dzswtl->db_mainoptions['translation__add_to_cart'] === '') {
        $dzswtl->db_mainoptions['translation__add_to_cart'] = esc_html__('Shop', DZSWTL_ID);
      }
    }
  }

  public static function sanitizeToPx($dimension) {
    if ($dimension !== '' && strpos($dimension, 'px') === false && strpos($dimension, 'auto') === false && strpos($dimension, 'em') === false) {
      return $dimension . 'px';
    }

    return $dimension;
  }

  public static function isDimensionInPx($dimension) {
    return !!(strpos($dimension, 'px') !== false) || !!(is_numeric($dimension));
  }

  /**
   * generate hardCodedKeys like cat
   * @param $original_pargs
   * @param $dzswtl
   * @return array
   */
  public static function generateHardCodedKeys($original_pargs, $dzswtl) {
    // -- analyze hard code here
    $hard_coded_possibilities = (DZSWTL_QUERY_PARAMS_HARDCODED_KEYS_POSSIBILITIES);
    $hard_coded_keys = array();


    foreach ($hard_coded_possibilities as $key) {
      if (isset($original_pargs[$key]) && $original_pargs[$key]) {
        $hard_coded_keys[$key] = $original_pargs[$key];
      }
    }

    if (count($hard_coded_keys)) {
      // -- *sideEffect
      $dzswtl->hard_coded_keys = $hard_coded_keys;
    }

    return $hard_coded_keys;
  }

  public static function generateHtmlWooCommerceMessage($productId) {
    global $dzswtl;

    $fout = '';


    if (function_exists('wc_get_product')) {

      $product = wc_get_product($productId);

      if ($product) {

        $name = $product->get_name();


        wc_get_page_id('cart');


        $fout .= '<div class="woocommerce-message from-dzswtl hide-after-5-secs" role="alert">
		<a href="' . get_permalink(wc_get_page_id('cart')) . '" tabindex="1" class="button wc-forward">' . esc_html__('View cart', DZSWTL_ID) . '</a> “' . $name . '” ' . esc_html__('has been added to your cart.', DZSWTL_ID) . '</div>';

      }

      $dzswtl->footer_extra_html .= '<script>document.addEventListener("DOMContentLoaded", function(){
document.querySelectorAll(\'.hide-after-5-secs\').forEach(function(el){setTimeout(function(){el.style.display=\'none\';}, 5000)})
});</script>';
    }


    return $fout;

  }

  public static function generateWtlSettingsForFrontend() {
    global $dzswtl;
    $fout = '';

    $fout .= '{ version: "' . DZSWTL_VERSION . '",site_url : "' . site_url() . '",ajaxurl : "' . admin_url('admin-ajax.php') . '",admin_url : "' . admin_url() . '"';


    $lab = 'filter_check_availability';
    if ($dzswtl->db_mainoptions[$lab] && $dzswtl->db_mainoptions[$lab] == 'on') {
      $fout .= ',' . $lab . ': "' . $dzswtl->db_mainoptions[$lab] . '"';
    }
    $lab = 'filter_check_availability_recalculate_count';
    if ($dzswtl->db_mainoptions[$lab] && $dzswtl->db_mainoptions[$lab] == 'on') {
      $fout .= ',' . $lab . ': "' . $dzswtl->db_mainoptions[$lab] . '"';
    }
    $lab = 'filter_check_availability_recalculate_count__hide_empty';
    if ($dzswtl->db_mainoptions[$lab] && $dzswtl->db_mainoptions[$lab] == 'on') {
      $fout .= ',' . $lab . ': "' . $dzswtl->db_mainoptions[$lab] . '"';
    }
    $lab = 'add_to_cart_modal';
    if ($dzswtl->db_mainoptions[$lab] && $dzswtl->db_mainoptions[$lab] == 'on') {
      $fout .= ',' . $lab . ': "' . $dzswtl->db_mainoptions[$lab] . '"';
    }
    $lab = 'filter_check_if_visible';
    if ($dzswtl->db_mainoptions[$lab] && $dzswtl->db_mainoptions[$lab] == 'on') {
      $fout .= ',' . $lab . ': "' . $dzswtl->db_mainoptions[$lab] . '"';
    }


    if (defined('DZSAP_VERSION')) {
      $fout .= ',zoomsounds_defined:"on"';
    } else {
      $fout .= ',zoomsounds_defined:"off"';
    }


    $arr_wishlist = DZSWTLHelpers::get_wishlist();
    $fout .= ' ,wishlist_ids:\'' . json_encode($arr_wishlist) . '\'';

    $fout .= '}';

    return $fout;

  }


  /**
   * @param $margs mutates
   */
  public static function sanitize_from_get_to_margs(&$margs) {


    $lab = 'orderby';
    if (isset($_GET[$lab])) {
      $margs[$lab] = urldecode($_GET[$lab]);
    }


    if (is_array($_GET) && count($_GET)) {

      foreach ($_GET as $lab => $val) {


        // -- we do not need this for now


        // -- let us remove this since we do not want to filter by attribute, but we need to sort by meta
      }
    }
  }


  public static function get_categories_for_select() {
    $terms = get_terms(DZSWTL_WOOCOMMERCE_PRODUCTS_CATEGORY, array(
      'hide_empty' => false,
    ));


    $foutarr = array(array(
      'label' => esc_html__('All categories', 'dzswtl'),
      'value' => '',
    ));

    foreach ($terms as $trm) {

      if ($trm && isset($trm->slug)) {
        array_push($foutarr, array(
          'label' => esc_attr($trm->name),
          'value' => esc_attr($trm->slug),
        ));
      } else {
        error_log('[dzswtl] .. slug does not exist - ' . print_r($trm, true) . ' .. terms - ' . print_r($terms, true));
      }
    }

    return $foutarr;

  }

  public static function get_cpt_posts() {

    $posts = get_posts([
      'post_type' => DZSWTL_POST_TYPE,
      'post_status' => 'publish',
      'numberposts' => -1
    ]);

    $final_arr = array();
    foreach ($posts as $po) {
      $aux = array(
        'label' => $po->post_title,
        'value' => $po->post_name,
        'term_id' => $po->ID,
      );
      array_push($final_arr, $aux);
    }
    return $final_arr;
  }

  /**
   * @param $argarr
   * @return array converts to array('type' => 'string','default' => $default,) for gutenberg php
   */
  public static function sanitize_config_to_gutenberg_register_block_type($argarr) {

    $foutarr = array();
    foreach ($argarr as $lab => $arr) {

      $key = $lab;
      $default = '';

      if (isset($arr['default'])) {
        $default = $arr['default'];
      }


      $foutarr[$key] = array(
        'type' => 'string',
        'default' => $default,
      );
    }
    return $foutarr;

  }

  public static function sanitize_config_to_gutenberg($arg) {
    // -- @ returns string
    $fout = '';
    $fout = $arg;
//    foreach ($arg as $key => $arr) {
////      $fout.=$key;
//      $default = '';
//
//      if (isset($arr['default'])) {
//        $default = $arr['default'];
//      }
//      $fout .= '\'' . $key . '\': {
//          \'type\': \'string\',
//          \'default\': \'' . $default . '\'
//        },';
//    }
    return $fout;
  }

  public static function sanitize_dzswtl_replaceGetFromMeta($argVal, $postId = '') {

    $fout = '';
    preg_match("/{{get_from_meta-(.*?)-(.*?)}}/", $argVal, $output_array);


    $default_val = '';
    $searched_val = '';

    if (isset($output_array[2])) {

      $default_val = $output_array[2];
    }

    if (isset($output_array[1])) {

      if (isset($postId) && $postId) {
        $po = get_post($postId);

        if ($po) {
          $searched_val = get_post_meta($po->ID, $output_array[1], true);
        }
      }
    }


    if ($searched_val) {

      $fout = $searched_val;
    } else {
      $fout = $default_val;
    }

    return $fout;
  }

  public static function sanitize_id_to_src($arg) {

    if (is_numeric($arg)) {

      $imgsrc = wp_get_attachment_image_src($arg, 'full');

      if (isset($imgsrc[0])) {

        return $imgsrc[0];
      }
    }


    return $arg;
  }


  public static function computeLayoutButtonLink($layoutDataItem, $product, $margs, $po) {
    global $dzswtl;

    $link = dzs_curr_url();

    if (isset($margs['page_url']) && $margs['page_url']) {
      $link = $margs['page_url'];
    }


    if (!isset($layoutDataItem['button_type'])) {
      $layoutDataItem['button_type'] = '';
    }
    if (!isset($layoutDataItem['button_functionality'])) {
      $layoutDataItem['button_functionality'] = '';
    }


    if ($layoutDataItem['button_type'] == DZSWTL_TABLE_BUTTON_TYPE['ADD_TO_CART'] && ($layoutDataItem['button_functionality'] == DZSWTL_TABLE_BUTTON_FUNCTIONALITY['LINK'])) {

      $link = dzs_remove_query_arg($link, 'dzswtl_action');
      $link = dzs_add_query_arg($link, 'add-to-cart', $po->ID);


      if ($product->is_type('variable') || $product->is_type('grouped')) {
        $link = get_permalink($po->ID);
      }
    }


    if (isset($layoutDataItem['button_functionality']) && $layoutDataItem['button_functionality'] == DZSWTL_TABLE_BUTTON_FUNCTIONALITY['DIRECT_LINK']) {
      $link = get_permalink($po->ID);
    }


    if ($layoutDataItem['button_type'] == DZSWTL_TABLE_BUTTON_TYPE['ADD_TO_CART']) {
      if ($dzswtl->db_mainoptions['purchasable_products_link_directly_to_checkout'] == 'on') {
        $link = wc_get_checkout_url();
        $link = dzs_add_query_arg($link, 'dzswtl-woo-only-one-item-in-cart', $po->ID);
      }

      if (DZSWTLHelpers::productIsDownloadable($po, true)) {
        $downloads = $product->get_downloads();
        foreach ($downloads as $key => $each_download) {
        }
        $link = site_url() . '?action=' . DZSWTL_QUERY_KEY_DOWNLOAD_TRACK . '&id=' . ($po->ID);
      }
      if ($product->is_type('simple')) {
        if (DZSWTLHelpers::woocommerceProductIsOutOfStock($product)) {
          $link = get_permalink(wc_get_page_id('shop'));
        }
      }

      if ($dzswtl->db_mainoptions['add_to_cart_links_to_login_for_unregistered'] == 'on') {
        if (function_exists('dzs_is_logged_in') && !dzs_is_logged_in()) {
          $link = DZSWTLHelpers::getLoginLink();
        }
      }

    }


    if (isset($layoutDataItem['button_functionality']) && $layoutDataItem['button_functionality'] == 'purchase_lightbox') {
      $link = '#';
    }


    $link = esc_url($link);


    if (($layoutDataItem['button_type'] == DZSWTL_TABLE_BUTTON_TYPE['MORE_BUTTON'] || $layoutDataItem['button_functionality'] == DZSWTL_TABLE_BUTTON_TYPE['NONE'])) {
      $link = '';
    }

    return $link;

  }

  /**
   * @return string
   */
  public static function getLoginLink() {
    global $dzswtl;

    $link = wp_login_url(dzs_curr_url());

    if ($dzswtl->db_mainoptions['redirect__login_page']) {
      $link = $dzswtl->db_mainoptions['redirect__login_page'];
    }

    return $link;
  }

  public static function enqueue_scriptAndStyle($script) {


    $cssUri = '';
    $jsUri = '';
    if ($script == 'spectrum') {
      $cssUri = DZSWTL_URL . 'libs/spectrum/spectrum.css';
      $jsUri = DZSWTL_URL . 'libs/spectrum/spectrum.js';
    }
    wp_enqueue_style('spectrum', $cssUri);
    wp_enqueue_script('spectrum', $jsUri);
  }

  public static function enqueue_ultibox() {

    wp_enqueue_script('ultibox', DZSWTL_URL . 'libs/ultibox/ultibox.js', array('jquery'), DZSWTL_VERSION);
    wp_enqueue_style('ultibox', DZSWTL_URL . 'libs/ultibox/ultibox.css');
  }

  public static function enqueue_dzsSelector() {

    wp_enqueue_script('dzssel', DZSWTL_URL . 'libs/dzsselector/dzsselector.js');
    wp_enqueue_style('dzssel', DZSWTL_URL . 'libs/dzsselector/dzsselector.css');


  }


  /**
   * called in layer button and in before extra html
   * @param string $extra_html <p>{{theid}} -> post id</p>
   * @param $productObject
   * @param WP_Post $po
   * @return array|string|string[]
   */
  public static function sanitizeExtraHtml($extra_html, $productObject = null, $po = null) {

    if (strpos($extra_html, '[ti_wishlists_addtowishlist') !== false) {
      global $product;
      if ($po && $productObject) {
        $product = $productObject->get_product($po->ID);
      }
    }


    if (function_exists('dzsap_get_download_link')) {
      if (strpos($extra_html, '{{downloadlink}}') !== false) {

        $extra_html = str_replace('{{downloadlink}}', dzsap_get_download_link($po, $po->ID), $extra_html);
      }
    }

    if (strpos($extra_html, 'dzsap-multisharer-but') !== false || strpos($extra_html, '{{audio_love_toggler_icon}}') !== false) {

      if (strpos($extra_html, 'dzsap-multisharer-but') !== false) {
        global $dzsap;
        if ($dzsap) {
          $dzsap->sw_enable_multisharer = true;
        }

        DZSWTLHelpers::enqueue_ultibox();
      }
      if (strpos($extra_html, '{{audio_love_toggler_icon}}') !== false) {
        $icon_wishlist = 'fa-heart-o';
        if (function_exists('dzsap_check_if_user_liked_track') && dzsap_check_if_user_liked_track($po->ID)) {
          $icon_wishlist = str_replace('fa-heart-o', 'fa-heart', $icon_wishlist);
        }
        $extra_html = str_replace('{{audio_love_toggler_icon}}', $icon_wishlist, $extra_html);
      }
    }


    $extra_html = str_replace('{{theid}}', $po->ID, $extra_html);
    $extra_html = str_replace('{{post_permalink}}', get_permalink($po->ID), $extra_html);

    if (strpos($extra_html, '{{addtocartlink}}') !== false) {

      $linkAddToCart = dzs_curr_url();
      $linkAddToCart = remove_query_arg('dzswtl_action', $linkAddToCart);
      $linkAddToCart = add_query_arg('add-to-cart', $po->ID, $linkAddToCart);
      $extra_html = str_replace('{{addtocartlink}}', $linkAddToCart, $extra_html);
    }



    if (strpos($extra_html, 'wishlist_star_toggler_icon') !== false) {


      $icon_wishlist = 'fa-star-o';
      if (DZSWTLHelpers::productIsInWishlist($po)) {
        $icon_wishlist = str_replace('fa-star-o', 'fa-star', $icon_wishlist);
      }
      $extra_html = str_replace('{{wishlist_star_toggler_icon}}', $icon_wishlist, $extra_html);
    }


    if (strpos($extra_html, '{{replacewithproductid}}') !== false) {

      if ($po->post_type == DZSWTL_WOOCOMMERCE_PRODUCT) {

        $extra_html = str_replace('{{replacewithproductid}}', $po->ID, $extra_html);
      } else {

        if ($po->post_type == 'dzsap_items') {

          if (get_post_meta($po->ID, 'dzsap_meta_productid', true)) {

            $extra_html = str_replace('{{replacewithproductid}}', get_post_meta($po->ID, 'dzsap_meta_productid', true), $extra_html);
          }
        }
      }
    }



    if (strpos($extra_html, '{{get_from_meta-') === 0) {
      $extra_html = DZSWTLHelpers::sanitize_dzswtl_replaceGetFromMeta($extra_html, $po->ID);
    }


    $extra_html = str_replace('{{hearts_svg}}', DZSWooTableHelper::export_svgs()['hearts'], $extra_html);
    $extra_html = str_replace('{{site_url}}', site_url(), $extra_html);


    $extra_html = str_replace('{{itempermalink}}', get_permalink($po->ID), $extra_html);

    return $extra_html;
  }

  public static function enqueue_script($type = 'main') {


    if ($type === 'fontawesome') {

      $fontawesomelink = DZSWTL_URL . 'libs/fontawesome/font-awesome.min.css';
      if (defined("DZSWTL_USE_FONTAWESOME_FROM_CDN") && DZSWTL_USE_FONTAWESOME_FROM_CDN == "ON") {
        $fontawesomelink = 'https://' . 'maxcdn' . 'bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css';
      }
      wp_enqueue_style('fontawesome', $fontawesomelink);
    }
    if ($type === 'filters') {

      wp_enqueue_script(DZSWTL_ID . '_filters', DZSWTL_URL . "libs/dzswootable/filters.js", array(), DZSWTL_VERSION);
    }
    if ($type === 'main') {

      wp_enqueue_style(DZSWTL_ID, DZSWTL_URL . "libs/dzswootable/dzswootable.css", array(), DZSWTL_VERSION);
      wp_enqueue_script(DZSWTL_ID, DZSWTL_URL . "libs/dzswootable/dzswootable.js", array('jquery'), DZSWTL_VERSION);
    }


    $cssUri = '';
    $jsUri = '';
    if ($type == 'spectrum') {
      $cssUri = DZSWTL_URL . 'libs/spectrum/spectrum.css';
      $jsUri = DZSWTL_URL . 'libs/spectrum/spectrum.js';
    }

    if ($cssUri) {
      wp_enqueue_style($type, $cssUri);
    }
    if ($jsUri) {
      wp_enqueue_script($type, $jsUri);
    }

  }

  public static function format_to_time($seconds) {
    return date('i:s', $seconds);

  }

  public static function checkInitQueryParams() {
    global $dzswtl;

    // -- *deprecated
    if (isset($_GET['dzswtl_action']) && $_GET['dzswtl_action'] == 'load_new_content') {
      // -- load new content
      error_reporting(0);
      add_filter('posts_search', array($dzswtl, 'filter_posts_search_include_tags'), 500, 2);
    }
    if (isset($_GET['dzswtl-woo-only-one-item-in-cart']) && $_GET['dzswtl-woo-only-one-item-in-cart']) {

      if (function_exists('WC')) {
        WC()->cart->empty_cart();
        WC()->cart->add_to_cart($_GET['dzswtl-woo-only-one-item-in-cart']);
      }
    }


  }


  public static function get_thumbnail_src($po, $im_size = 'full') {


    $image = '';


    $image = wp_get_attachment_image_src(get_post_thumbnail_id($po->ID), $im_size, false);


    if (isset($image[0])) {
      $image = $image[0];
    }


    if ($image == '') {
      if ($po->post_type == 'dzsap_items') {
        $image = get_post_meta($po->ID, 'dzsap_meta_item_thumb', true);
      }
    }

    return $image;
  }


}

function dzswtl_handle_save_term_meta($term_id, $tt_id) {
  global $dzswtl;
  foreach ($dzswtl->attribute_extra_types as $type => $label) {
    if (isset($_POST[$type])) {
      update_term_meta($term_id, $type, $_POST[$type]);
    }
  }
}