'use strict';
import {$es, documentReady} from './js_common/_esjquery';
import {add_query_arg, get_query_arg, get_query_arg_all, isDomElementVisible} from "./js_common/_dzs_helpers";
import {
  DZSWTL_FILTERS_SINGLETON,
  DZSWTL_MODEL_SHOP_INITIALLY_SETUP,
  DZSWTL_TABLE_FILTERS_CURRENT_ARGUMENTS_KEY,
  DZSWTL_TABLE_FILTERS_USER_SET_FILTERS,
  DZSWTL_VIEW_AJAX_MAIN_SHORTCODE_CONTAINER,
  DZSWTL_VIEW_HTML_CLASS_MAIN_CONTAINER
} from "./config/_dzswtl_constants";
import {isInfiniteScroll} from "./js_dzswtl/_ajax_loadNewPage";
import {ajax_loadNewFilterCount} from "./js_dzswtl/_ajax_loadNewFilterCount";


class FiltersSingleton {
  /** @property {FiltersSingleton} */
  selfInstance = null;

  /** @property {HTMLElement} */
  $shopTarget = null;

  /** @property {DzsWooTable} */
  $shopTargetInstance = null;

  /** @property {Object|null} available filters on page */
  availableFilters = null;

  /** @property {Object|null} available filters on page */
  actualSearch_forceUpdateFilters = null;

  /** @property {Object|null} available filters on page */
  hardCodedKeys = null;


  /** @property {boolean|null} available filters on page */
  is_recalculate_attribute_data = false;

  isInited = false;


  constructor() {
    this.selfInstance = this;
    this.classInit();
  }

  classInit() {


    this.initOnShopLoaded().then(() => {
      this.reinit();
    })

    this.api_reinit = this.reinit;


  }

  /**
   * called in dzsWooTable - wait for filters to load
   * -> before calculating payload
   */
  handleAjaxContentLoading() {
    this.aggregateAllFilters();

    this.mapToUserSetFilters()
  }

  /**
   * called externally
   * table_paged and sort_by comes from query
   * other filters are set from the page on aggregation
   */
  handleAjaxContentLoaded(arrLoadNewResponse) {

    let newUrlLoadedContent = window.location.href;
    const selfClass = this;


    var isArgPostsPerPageChanged = false,
      isArgLayoutChanged = false
    ;


    try {

      const userSetFilters = window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS];


      if (arrLoadNewResponse.paged === '1' || arrLoadNewResponse.paged === '' || arrLoadNewResponse.max_pages === 1 || arrLoadNewResponse.max_pages === 0) {
        if (this.availableFilters['table_paged']) {
          this.availableFilters['table_paged'].value = '';
          this.availableFilters['table_paged'].defaultValue = '';
        }
        delete userSetFilters['table_paged'];
        newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'table_paged', 'NaN');
      } else {

        userSetFilters['table_paged'] = arrLoadNewResponse.paged
      }

      if (arrLoadNewResponse.userSet_sort_by !== undefined) {

        const {userSet_sort_by} = arrLoadNewResponse;
        if (userSet_sort_by) {

          newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'sort_by', userSet_sort_by);
        }

        if (userSet_sort_by === '') {

          newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'sort_by', 'NaN');
        }
      }

      Object.keys(userSetFilters).forEach(queryKey => {
        const queryVal = userSetFilters[queryKey];

        if (queryVal) {
          newUrlLoadedContent = add_query_arg(newUrlLoadedContent, queryKey, queryVal);
        } else {
          newUrlLoadedContent = add_query_arg(newUrlLoadedContent, queryKey, 'NaN');
        }
      })


      if (!isArgPostsPerPageChanged) {
        newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'posts_per_page', 'NaN');
      }

      if (!isArgLayoutChanged) {
        newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'layout', 'NaN');
      }


      if (isInfiniteScroll(selfClass.$shopTarget)) {
        newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'paged', 'NaN');
      }

      if (selfClass.hardCodedKeys && Object.keys(selfClass.hardCodedKeys).length) {
        Object.keys(selfClass.hardCodedKeys).forEach(function (keyLabel) {
          add_query_arg(newUrlLoadedContent, keyLabel, 'NaN')
        })
      }

      if (newUrlLoadedContent.indexOf('?') === -1 && newUrlLoadedContent.indexOf('&') > -1) {
        newUrlLoadedContent = newUrlLoadedContent.replace('&', '?');
      }

      if (get_query_arg(newUrlLoadedContent, 'subcat_slug')) {

        newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'subcat', get_query_arg(newUrlLoadedContent, 'subcat_slug'));
        newUrlLoadedContent = add_query_arg(newUrlLoadedContent, 'subcat_slug', 'NaN');
      }


      if (newUrlLoadedContent && newUrlLoadedContent !== window.location.href) {
        var stateObj = {foo: 0, href: newUrlLoadedContent};
        history.pushState(stateObj, null, newUrlLoadedContent);
      }
    } catch (err) {
      console.log(err);
    }
  }

  initOnShopLoaded() {
    return new Promise((success, reject) => {
      let inter_check = 0;
      inter_check = setInterval(() => {
        if (window[DZSWTL_MODEL_SHOP_INITIALLY_SETUP]) {
          clearInterval(inter_check);
          success();
        }
      }, 10);
    })
  }

  getShopTarget() {


    const $shopTargets = $es('.' + DZSWTL_VIEW_AJAX_MAIN_SHORTCODE_CONTAINER)
    if (!$shopTargets.length()) {
      this.$shopTarget = $es('.' + DZSWTL_VIEW_HTML_CLASS_MAIN_CONTAINER).$el;
    } else {
      this.$shopTarget = $shopTargets.$el;
    }

    if (this.$shopTarget) {
      this.$shopTargetInstance = this.$shopTarget.SelfInstance;
    }


    if (this.$shopTargetInstance) {
      if (this.$shopTargetInstance.cthis.attr('data-recalculate_attribute_data') === 'on') {
        this.selfInstance.is_recalculate_attribute_data = true;
      }
    }

  }

  /**
   * called on reinit
   */
  getShopTargetAttributes() {

    if (!(this.$shopTarget && this.$shopTarget.SelfInstance)) {
      this.$shopTarget = document.querySelector('.' + DZSWTL_VIEW_HTML_CLASS_MAIN_CONTAINER)
    }

    if (this.$shopTarget && this.$shopTarget.SelfInstance) {

      const shopTargetCthis = this.$shopTarget.SelfInstance.cthis;

      if (shopTargetCthis.attr('data-actual_search_forceupdatefilters')) {
        this.actualSearch_forceUpdateFilters = JSON.parse(shopTargetCthis.attr('data-actual_search_forceupdatefilters'));
        shopTargetCthis.attr('data-actual_search_forceupdatefilters', '');
      }

      if (shopTargetCthis.attr('data-count-all-number')) {
        this.view_updateCountAllFilterNumber(shopTargetCthis.attr('data-count-all-number'));
      }


      if (shopTargetCthis.attr('data-hard-coded-keys')) {
        this.hardCodedKeys = JSON.parse(shopTargetCthis.attr('data-hard-coded-keys'));
      }
    } else {
    }
  }

  /**
   * available filters + search
   */
  getAvailableFilters() {

    this.availableFilters = {};
    $es('.dzs-woo-feed-filter-obj').each(($el) => {

      const feedLab = $el.attr('data-dzs-woo-feed-lab');
      const feedType = $el.attr('data-dzs-woo-feed-type');

      if (feedLab) {

        let initialValue = '';

        let $inputElements = $es('*[name="' + feedLab + '"]');
        if (!$inputElements.length()) {
          // -- checkbox
          $inputElements = $es('*[name="' + feedLab + '[]"]')
        }

        this.availableFilters[feedLab] = {

          type: feedType,
          value: initialValue,
          defaultValue: '',
          '$element': $inputElements,
        }
      }
    })
    $es('.woo-table-searcher').each(() => {

      const feedLab = 'the_search';
      const feedType = 'search';

      let initialValue = '';

      const $searchEl = $es('*[name="' + feedLab + '"],*[name="' + 'woo_search' + '"]');


      this.availableFilters[feedLab] = {
        type: 'input',
        value: initialValue,
        defaultValue: '',
        '$element': $searchEl,
      }
    })


    const getQueryAll = get_query_arg_all(window.location.href);
    Object.keys(getQueryAll).forEach((queryKey) => {
      const feedLab = queryKey;
      const initialValue = getQueryAll[queryKey];


      if (queryKey === 'sort_by') {
        return;
      }
      if (!this.availableFilters[feedLab]) {
        this.availableFilters[feedLab] = {
          type: 'from_query',
          value: initialValue,
          defaultValue: '',
          '$element': null,
        }
      }
    })


    delete this.availableFilters['search'];

    this.aggregateAllFilters();
  }

  /**
   * called on loading and on reinit
   */
  mapToUserSetFilters() {


    const userSetFilters = {};
    if(this.isInited){

      Object.keys(this.availableFilters).forEach((key) => {
        const availableFilterObj = this.availableFilters[key];
        userSetFilters[key] = availableFilterObj.value;
      });

      if (typeof userSetFilters.table_paged !== 'undefined') {
        delete userSetFilters.table_paged;
      }


      const currentFilterArgs = window[DZSWTL_TABLE_FILTERS_CURRENT_ARGUMENTS_KEY] ? window[DZSWTL_TABLE_FILTERS_CURRENT_ARGUMENTS_KEY] : {};

      window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS] = userSetFilters;
      window[DZSWTL_TABLE_FILTERS_CURRENT_ARGUMENTS_KEY] = Object.assign(currentFilterArgs, window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS]);
    }
  }

  reinit() {

    const selfInstance = this;

    selfInstance.isInited = true;

    // -- other plugins
    if (window['dzs_initSliders']) {
      window['dzs_initSliders']();
    }


    this.getShopTarget();
    this.getAvailableFilters();
    this.getShopTargetAttributes();
    this.mapToUserSetFilters();

    if (this.is_recalculate_attribute_data) {
      try {
        window.dzswtl_product_ids_that_fit_query = JSON.parse($es('.feed-dzswtl_product_ids_that_fit_query').eq(0).attr('data-attdata'));
      } catch (err) {
        console.log('err parsing - ', err);
      }
      // -- recalculate attribute data to adjust filters
    }

    if (window.dzswtl_settings && window.dzswtl_settings['filter_check_availability_recalculate_count__hide_empty'] === 'on') {

      jQuery('body').addClass('dzswtl-filters--hide-empty');
    } else {

      jQuery('body').removeClass('dzswtl-filters--hide-empty');
    }

    var selectorAjaxTriggers = '.dzs-woo-ajax';
    jQuery(document).on('click.dzswtl', selectorAjaxTriggers + ',.dzs-woo-table-sort-input,.delete-filter, .clear-all-filters', this.view_handleMouse(this));


    Object.keys(this.availableFilters).forEach((key) => {
      const availableFilterObj = this.availableFilters[key];

      if (availableFilterObj.$element) {

        availableFilterObj.$element.$els.forEach(($el) => {
          $el.addEventListener('change', selfInstance.view_handleChange(selfInstance));
        })
      }
    })

    this.reinit_getCurrentIdsOnPage();
  }

  reinit_getCurrentIdsOnPage() {
    // -- called on reinit
    window.dzswtl_current_ids_on_page = [];


    $es('.dzswtl--item-instance').each(function ($esItem) {
      var _t = $esItem;
      if (_t.attr('data-post-id')) {
        window.dzswtl_current_ids_on_page.push(_t.attr('data-post-id'));
      }
    })


    if (window.dzswtl_settings && window.dzswtl_settings['filter_check_availability'] === 'on') {
      if (this.$shopTargetInstance) {
        ajax_loadNewFilterCount(this.$shopTargetInstance);
      }
    }
  }

  view_handleChange(selfInstance) {
    return (e) => {
      selfInstance.aggregateAllFilters();
    }
  }

  /**
   * calculate the values of filters
   * table_paged and sort_by comes from query
   * other filters are set from the page on aggregation
   */
  aggregateAllFilters() {
    const selfInstance = this;

    /**
     *
     * @param {HTMLElement} $el_
     * @param {number} maxLevels
     * @return {HTMLElement}
     */
    const getParentElement = ($el_, maxLevels) => {
      let $parent_ = $el_;
      let currIndex = 0;
      while ($parent_ = $parent_.parentElement) {

        if (currIndex >= maxLevels) {
          break;
        }
        currIndex++;
      }
      return $parent_;
    }

    /**
     *
     * @param {HTMLElement} $el_
     * @param {boolean|undefined} isForcingReturnTrue
     * @returns {boolean}
     */
    const isElementRequiredVisible = ($el_, isForcingReturnTrue) => {
      if (isForcingReturnTrue) {
        return true;
      }
      if (window.dzswtl_settings.filter_check_if_visible === 'on') {

        if ($el_.classList.contains('dzs-style-me')) {
          return true;
        }

        const $maybeCheckboxLabel = getParentElement($el_, 0);

        if ($maybeCheckboxLabel && $maybeCheckboxLabel.nodeName === 'LABEL') {
          if (isDomElementVisible($maybeCheckboxLabel)) {
            return true;
          }
        }

        const $maybeDzsSelectWrapper_ = getParentElement($el_, 3);

        if ($maybeDzsSelectWrapper_ && $maybeDzsSelectWrapper_.classList.contains('dzs-select-wrapper')) {
          if (isDomElementVisible($maybeDzsSelectWrapper_)) {
            return true;
          }
        }

        if (!isDomElementVisible($el_)) {
          return false;
        }
      }

      return true;
    }

    if(selfInstance.isInited && selfInstance.availableFilters){
      Object.keys(selfInstance.availableFilters).forEach((key) => {
        const availableFilterObj = selfInstance.availableFilters[key];

        let filterValue = '';
        if (availableFilterObj.type === 'multiple') {
          filterValue = [...availableFilterObj.$element.$els].filter(($el) => isElementRequiredVisible($el, undefined) && $el.checked).map(($el) => $el.value).join(',');
        }
        if (availableFilterObj.type === 'select' || availableFilterObj.type === 'input' || availableFilterObj.type === 'slider') {
          const $theEl = availableFilterObj.$element.$el;

          const isForcingReturnTrue = availableFilterObj.type === 'slider';
          if ($theEl) {
            if (isElementRequiredVisible($theEl, isForcingReturnTrue)) {
              filterValue = $theEl.value;
            }
          }
        }
        if (availableFilterObj.type === 'from_query') {
          filterValue = get_query_arg(window.location.href, key);
          if (availableFilterObj.defaultValue) {
            if (!filterValue) {
              filterValue = availableFilterObj.defaultValue;
            }
          }
        }


        availableFilterObj.value = filterValue;
      })

    }

  }


  /**
   * call from anyonere
   * recalculate based on window.dzswtl_filter_referenceTaxonomyCount
   */
  filters_recalculateFilterCount() {

    const calculateFilterCountForTerm = (termKey, productsInTerm) => {
      let nrProductsInTerm = 0;
      productsInTerm.forEach((productIdInTerm) => {
        if (window.dzswtl_product_ids_that_fit_query && window.dzswtl_product_ids_that_fit_query.indexOf(productIdInTerm) > -1) {
          nrProductsInTerm++;
        }
      })

      const $theActualCount = $es('*[data-term_id="' + termKey + '"]').find('.the-actual-count');
      $theActualCount.html(nrProductsInTerm);

      if ($theActualCount.parent().parent().parent().hasClass('bigoption')) {
        $theActualCount.parent().parent().parent().attr('data-the-count', nrProductsInTerm);
      }
    }

    if (window.dzswtl_filter_referenceTaxonomyCount) {

      Object.keys(window.dzswtl_filter_referenceTaxonomyCount).forEach(function (taxonomyKey) {
        if (window.dzswtl_filter_referenceTaxonomyCount[taxonomyKey]) {
          for (const [termKey, productsInTerm] of Object.entries(window.dzswtl_filter_referenceTaxonomyCount[taxonomyKey])) {
            calculateFilterCountForTerm(termKey, productsInTerm);
          }
        }

      })
    } else {
      console.log('[dzswtl] filter_count undefined')
    }


  }

  clearAll() {

    const selfInstance = this;
    Object.keys(this.availableFilters).forEach((key) => {
      const availableFilterObj = this.availableFilters[key];

      selfInstance.clearFilterFromAggregatedFilters(key);


      if (window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS] && window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS][key] !== undefined) {
        window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS][key] = '';
      }

    });
  }


  /**
   *
   * @param {esJquery} $filterForm
   */
  viewFormClearValue($filterForm, isGoingToTriggerEvent) {


    if ($filterForm.get(0) && $filterForm.get(0).nodeName === 'INPUT') {
      if ($filterForm.attr('type') === 'checkbox') {
        $filterForm.get(0).checked = false;
      } else {

        $filterForm.get(0).value = '';
        $filterForm.val('');
      }
    }

    if (isGoingToTriggerEvent) {

      $filterForm.trigger('change');
    }

    if ($filterForm.get(0)) {
      let $elementTryToReinitSelector = null;
      if ($filterForm.get(0).nodeName === 'INPUT') {

        if ($filterForm.get(0) && $filterForm.get(0).parent_selector_wrapper && $filterForm.get(0).parent_selector_wrapper.get(0)) {
          $elementTryToReinitSelector = $filterForm.get(0).parent_selector_wrapper.get(0);
        }
      }
      if ($filterForm.get(0).nodeName === 'SELECT') {
        var elements = $filterForm.get(0).options;
        for (var i = 0; i < elements.length; i++) {
          elements[i].selected = false;
        }


        if ($filterForm.get(0)) {
          $elementTryToReinitSelector = $filterForm.get(0);
        }

      }

      if ($elementTryToReinitSelector && $elementTryToReinitSelector.api_recheck_value_from_input) {
        $elementTryToReinitSelector.api_recheck_value_from_input();

      }
    }

  }

  clearFilterFromAggregatedFilters(filterKey) {

    let filterObj = null;
    const selfInstance = this;
    if (selfInstance.availableFilters[filterKey]) {
      filterObj = selfInstance.availableFilters[filterKey];
    }


    // -- selects

    var $filterForm = jQuery('select[name="' + filterKey + '"]').eq(0);

    if (filterKey === 'the_search') {
      $filterForm = jQuery('*[name="' + 'woo_search' + '"]').eq(0);
      window[DZSWTL_TABLE_FILTERS_CURRENT_ARGUMENTS_KEY] = JSON.stringify(Object.assign(JSON.parse(window[DZSWTL_TABLE_FILTERS_CURRENT_ARGUMENTS_KEY]), {search: ''}));
    }

    if (filterObj) {
      filterObj.value = '';
      filterObj.defaultValue = '';
      if (filterObj.type === 'from_query') {
        const newUrl = add_query_arg(window.location.href, filterKey, 'NaN');
        var stateObj = {event: 'no-event', href: newUrl};
        history.pushState(stateObj, null, newUrl);
      }
      if (filterObj.$element) {
        $filterForm = filterObj.$element;
      }


      if ($filterForm) {
        if (filterObj.type === 'multiple') {
          $filterForm.each($filterCheckbox => {
            selfInstance.viewFormClearValue($filterCheckbox, false);
          })


        }
        if (filterObj.type === 'select' || filterObj.type === 'input' || filterObj.type === 'slider') {


          selfInstance.viewFormClearValue($filterForm, false);

          if (filterObj.type === 'slider') {
            if ($filterForm.get(0) && $filterForm.get(0).$slider) {
              $filterForm.get(0).sliderUpdateFromInput($filterForm.get(0).$slider, $es($filterForm));
            }
          }
        }

      }

    }
  }

  sendLoadNewContentToShop(loadNewContentArgs) {

    if (this['hardCodedKeys'] && Object.keys(this['hardCodedKeys']).length) {
      for (const key in this['hardCodedKeys']) {
        loadNewContentArgs[key] = this['hardCodedKeys'][key];
      }
    }
    this.$shopTarget.SelfInstance.load_new_content(loadNewContentArgs);
  }

  /**
   *
   * @param {FiltersSingleton} selfInstance
   * @returns {(function(*): (boolean|undefined))|*}
   */
  view_handleMouse(selfInstance) {
    return function (e) {


      var $t = jQuery(this);

      if (e.type === 'click') {

        if ($t.hasClass('dzs-woo-ajax')) {

          const theHref = $t.attr('href');
          const theHrefKeys = get_query_arg_all(theHref);


          const newArgs = get_query_arg_all(theHref);


          if ($t.hasClass('pagination-link')) {
            newArgs.caller_event = 'PAGINATION_EVENT';
            window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS].table_paged = newArgs.table_paged;
          } else {

            Object.keys(theHrefKeys).forEach(hrefKey => {

              if (hrefKey === 'sort_by') {
                return;
              }


              if (selfInstance.availableFilters[hrefKey] && selfInstance.availableFilters[hrefKey].type !== 'from_query') {

                // todo: treat this case better
              } else {

                selfInstance.availableFilters[hrefKey] = {
                  type: 'from_query',
                  value: theHrefKeys[hrefKey],
                  defaultValue: theHrefKeys[hrefKey],
                  '$element': null,
                }
              }
            })
          }

          selfInstance.sendLoadNewContentToShop(newArgs);


          e.stopImmediatePropagation();
          e.stopPropagation();
          e.preventDefault();
          $t.get(0).dzs_stop_propagation = true;
          return false;
        }

        if ($t.hasClass('delete-filter')) {


          // -- delete filter

          let args = {};

          const filterKey = $t.attr('data-lab_full');
          let filterObj = null;

          args[filterKey] = '';

          selfInstance.clearFilterFromAggregatedFilters(filterKey);


          if (window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS] && window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS][filterKey] !== undefined) {
            window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS][filterKey] = '';
          }

          args.caller_event = 'DELETE_SINGLE_FILTER';

          selfInstance.sendLoadNewContentToShop((args));
          $t.parent().fadeOut('fast');


          return false;
        }


        /**
         * clear all filters
         */
        if ($t.hasClass('clear-all-filters')) {


          // -- delete filter
          let loadNewContentArgs = {};


          // -- selects

          jQuery('.filtered-by-con').children().addClass('animate-filter-out');

          loadNewContentArgs.call_from = 'clear-all-filters';

          selfInstance.clearAll();

          if (window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS]) {
            Object.keys(window[DZSWTL_TABLE_FILTERS_USER_SET_FILTERS]).forEach((key) => {
              loadNewContentArgs[key] = '';
            })
          }
          loadNewContentArgs['sort_by'] = '';
          selfInstance.view_clearAllSortControls();

          loadNewContentArgs.caller_event = 'DELETE_ALL_FILTERS';

          selfInstance.sendLoadNewContentToShop(loadNewContentArgs);


          return false;
        }


        if ($t.hasClass('dzs-woo-table-sort-input')) {
          var args = {};

          let sortByVal = '';

          if ($t.parent().hasClass('sorting-asc') || $t.parent().hasClass('sorting-desc')) {


            sortByVal = '';
            $t.parent().removeClass('sorting-asc sorting-desc');
          } else {

            selfInstance.view_clearAllSortControls();

            sortByVal += $t.attr('data-sort-lab');

            if ($t.hasClass('sort-asc')) {
              sortByVal += '{asc}';

              $t.parent().addClass('sorting-' + 'asc');
            }
            if ($t.hasClass('sort-desc')) {
              sortByVal += '{desc}';

              $t.parent().addClass('sorting-' + 'desc');
            }
          }
          args['sort_by'] = sortByVal;
          args['call_from'] = 'sort-input';
          let loadNewContentArgs = {};
          loadNewContentArgs.caller_event = 'SORT_INPUT';

          selfInstance.sendLoadNewContentToShop(args);

        }

      }
    }
  }

  /**
   * clear all sort controls from tables
   */
  view_clearAllSortControls() {

    $es(this.$shopTarget).find('.sort-controls').each(($sortInput) => {
      $sortInput.removeClass('sorting-asc')
      $sortInput.removeClass('sorting-desc')
    })
  }


  view_updateCountAllFilterNumber(newNumber) {

    $es('.dzswtl-filter--count--placeholder').html('(' + newNumber + ')');
  }
}

documentReady(() => {


  if (!window[DZSWTL_FILTERS_SINGLETON]) {
    window[DZSWTL_FILTERS_SINGLETON] = new FiltersSingleton();
  }

})


