import {loadScriptIfItDoesNotExist} from "../../../js_common/_dzs_helpers";
import {$es, documentReady} from "../../../js_common/_esjquery";

window.trigger_change_on_target = function (arg) {
  arg.trigger('change');
}

const sanitizeToValueLabel = (arg, isSanitizeToTime) => {
  if (isSanitizeToTime) {
    var s = Math.round(arg);
    var m = 0;
    var h = 0;
    if (s > 0) {
      while (s > 3599 && s < 3000000 && isFinite(s)) {
        h++;
        s -= 3600;
      }
      while (s > 59 && s < 3000000 && isFinite(s)) {
        m++;
        s -= 60;
      }
      if (h) {

        return String((h < 10 ? "0" : "") + h + ":" + String((m < 10 ? "0" : "") + m + ":" + (s < 10 ? "0" : "") + s));
      }
      return String((m < 10 ? "0" : "") + m + ":" + (s < 10 ? "0" : "") + s);
    } else {
      return "00:00";
    }
  }
  return arg;
}


window['dzs_initSliders'] = function () {

  const setHtmlOfHandles = (_t, paramMin, paramMax) => {

    var handle1 = _t.find(".handle-min");
    handle1.html('<div class="center-text">' + sanitizeToValueLabel(paramMin) + '</div>');
    var handle2 = _t.find(".handle-max");
    handle2.html('<div class="center-text">' + sanitizeToValueLabel(paramMax) + '</div>');
  }

  /**
   * side effects
   * @param $slider
   * @param $input
   * @returns {{valueMax: (number|number), valueMin: (number|number), SEPARATOR: (string|string), paramMax: (number|number), paramMin: (number|number)}}
   */
  const sliderUpdateFromInput = ($slider, $input) => {

    const SEPARATOR = $slider.attr('data-separator') ? String($slider.attr('data-separator')) : 'wtn-wtn'

    const paramMin = $slider.attr('data-slider-min') ? Number($slider.attr('data-slider-min')) : 0;
    const paramMax = $slider.attr('data-slider-max') ? Number($slider.attr('data-slider-max')) : 100;


    const sliderInputValue = $input.val();

    let valueMin = paramMin;
    let valueMax = paramMax;
    if (sliderInputValue.indexOf(SEPARATOR) > -1) {
      [valueMin, valueMax] = sliderInputValue.split(SEPARATOR);
    }

    setHtmlOfHandles($slider, valueMin, valueMax);

    if ($slider.hasClass('inited')) {
      jQuery($slider.get(0)).slider("values", [valueMin, valueMax]);
    }

    return {
      SEPARATOR,
      valueMin,
      valueMax,
      paramMin,
      paramMax,
    }
  }

  $es('.dzs-slider-auto-init').each(function ($slider) {
    const $input = $slider.prev();
    const {
      SEPARATOR,
      valueMin,
      valueMax,
      paramMin,
      paramMax,
    } = sliderUpdateFromInput($slider, $input);


    if (!$slider.hasClass('inited')) {
      $input.get(0).$slider = $slider;
      $input.get(0).sliderUpdateFromInput = sliderUpdateFromInput;

      if (window.jQuery && window.jQuery.fn.slider) {
        jQuery($slider.get(0)).slider({
          range: true,
          min: paramMin,
          max: paramMax,
          step: 1,
          values: [valueMin, valueMax],
          slide: function (e, ui) {
            var _t = jQuery(e.target);
            setHtmlOfHandles(_t, ui.values[0], ui.values[1]);

            _t.prev().val(ui.values[0] + SEPARATOR + ui.values[1]);
            if (window.trigger_change_on_target_inter) {
              clearTimeout(window.trigger_change_on_target_inter);
            }
            window.trigger_change_on_target_inter = setTimeout(function () {
              window.trigger_change_on_target(_t.prev());
            }, 300);
          }
        });
      }

      $slider.addClass('inited');

    }
  })
}

documentReady(() => {


  loadScriptIfItDoesNotExist('', 'jQuery').then((res) => {


    (() => {
      return new Promise((resolve, reject) => {
        let inter_checkSlider = 0;
        inter_checkSlider = setInterval(() => {
          if (window.jQuery.fn.slider && window.jQuery.widget && window.jQuery.ui) {
            resolve('sliderReady');
            clearInterval(inter_checkSlider);
          }
        }, 50);
      })
    })().then((resSlider) => {
      window['dzs_initSliders']();
    });
  }).catch((err) => {
      console.log('err - ', err);
    }
  )
})